from restful_lib import Connection, ConnectionError
from microblog_exceptions import MicroBlogMsgLimitExceeded

class FailWhale(Exception):
    def __str__(self):
        return "Oh noes, the failwhale cometh!"

TWITTER_ENDPOINT = "http://twitter.com/statuses/update.json"

class Twitter(object):
    def __init__(self, username,password):
        self._conn = Connection(TWITTER_ENDPOINT, username, password)

    def post(self, message, clip_length=False):
        if isinstance(message, list):
            # Default to a joining up the list of words with a space
            message = " ".join(message)

        if len(message)>140:
            if clip_length:
                message = message[:140]
            else:
                raise MicroBlogMsgLimitExceeded

        resp = c.request_post("", args={"status":message})

        if resp.get('headers').get('status') not in ["200", 200, "204", 204]:
            raise FailWhale



import warnings

from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned, FieldError
from django.conf import settings
from django.db.models.query import QuerySet
from django.core.paginator import Paginator, InvalidPage, EmptyPage


from utils import rc, has_model, flatten_dict
from responses import *

class BaseHandler(object):
	"""
	Basehandler that gives you CRUD for free.
	You are supposed to subclass this for specific
	functionality.
	
	All CRUD methods (`read`/`update`/`create`/`delete`)
	receive a request as the first argument from the
	resource. Use this for checking `request.user`, etc.
	"""
	#__metaclass__ = HandlerMetaClass
	
	allowed_methods = ('GET', 'POST', 'PUT', 'DELETE')
	anonymous = is_anonymous = False
	exclude = ( 'id', )
	fields =  ( )
	
	def flatten_dict(self, dct):
		return flatten_dict(dct)
	
	def has_model(self):
		return hasattr(self, 'model') or hasattr(self, 'queryset')

	def queryset(self, request):
		return self.model.objects.all()
	
	def value_from_tuple(tu, name):
		for int_, n in tu:
			if n == name:
				return int_
		return None
	
	def _querydict_to_dict(self, q):
		"""
		Use on e.g. request.GET in order to get a python dict suitable for **kwargs-ing into a queryset filter.
		"""
		d = {}
		for k in q.keys():
			d[str(k)] = q[k]	# str() is necessary, apparently - presumably because normal kwargs are ascii
		return d
		
	def exists(self, **kwargs):
		if not self.has_model():
			raise NotImplementedError
		
		try:
			self.model.objects.get(**kwargs)
			return True
		except self.model.DoesNotExist:
			return False
	
	@has_model
	def read(self, request, format, *args, **kwargs):

		pkfield = self.model._meta.pk.name

		if pkfield in kwargs:
			try:
				return self.queryset(request).get(pk=kwargs.get(pkfield))
			except ObjectDoesNotExist:
				return HttpResponseNotFound('The object you requested could not be found')
			except MultipleObjectsReturned: # should never happen, since we're using a PK
				return HttpResponseBadRequest('More than one object was found!')
		else:
			return self.queryset(request).filter(*args, **kwargs)
	
	@has_model
	def create(self, request, format, *args, **kwargs):
		attrs = self.flatten_dict(request.POST)
		
		try:
			inst = self.queryset(request).get(**attrs)
			return HttpResponseDuplicateEntry('This entry already exists!')
		except self.model.DoesNotExist:
			inst = self.model(**attrs)
			inst.save()
			return inst
		except self.model.MultipleObjectsReturned:
			return HttpResponseDuplicateEntry('This entry already exists, and more than once to boot!')
	
	@has_model
	def update(self, request, format, *args, **kwargs):
		pkfield = self.model._meta.pk.name

		if pkfield not in kwargs:
			# No pk was specified
			return HttpResponseBadRequest('Please specify a unique identifier for the entry to update')

		try:
			inst = self.queryset(request).get(pk=kwargs.get(pkfield))
		except ObjectDoesNotExist:
			return HttpResponseBadRequest('The object you requested to update could not be found')
		except MultipleObjectsReturned: # should never happen, since we're using a PK
			return HttpResponseBadRequest('The unique identifier you supplied returned more than one entry!')

		attrs = self.flatten_dict(request.POST)
		for k,v in attrs.iteritems():
			setattr( inst, k, v )

		inst.save()
		return HttpResponseAllOK()
	
	@has_model
	def delete(self, request, format, *args, **kwargs):
		try:
			inst = self.queryset(request).get(*args, **kwargs)

			inst.delete()

			return HttpResponseDeleted()
		except self.model.MultipleObjectsReturned:
			return HttpResponseDuplicateEntry('The unique identifier you supplied returned more than one entry!')
		except self.model.DoesNotExist:
			return HttpResponseGone('This entry already does not exist')
		
class AnonymousBaseHandler(BaseHandler):
	"""
	Anonymous handler.
	"""
	is_anonymous = True
	allowed_methods = ('GET',)

"""
The following two classes subclass BaseHandler to form classes for handling
the django Model in two different ways: as a collection (CollectionHandler)
and as a member (MemberHandler).

A collection basically corresponds to a queryset, e.g. Person.objects.all(),
whereas a member corresponds to a single Model instance, e.g. "Joe Bloggs".

An easy analogy can be drawn with Django's generic object_list (a
collection) and object_detail (a member).

A pretty good article on collections and members: <http://www.projectzero.org/sMash/1.0.x/docs/zero.devguide.doc/zero.resource/rest-api.html>
For a table of how HTTP methods are interpreted for collections and members: <http://en.wikipedia.org/wiki/REST#RESTful_web_services>
"""

class CollectionHandler(BaseHandler):
	"""
	This class implements appropriate definitions of read, create, update,
	and delete for the interpretation of the resource as a COLLECTION.
	"""
	
	class ClientQueryVocabulary:
		"""
		Represents a vocabulary used by a client to indicate things like pagination and sorting.
		There seems to be no standardization here, so for every different possible client we create a vocab.
		"""
		def __init__(self,
				paginate_page_name,					# Name of the GET attribute indicating which page we want
				paginate_results_per_page_name,		# Name of the GET attribute indicating how many results we want per page
				
				sort_field_name,					# Name of the GET attribute indicating which column/field to sort by
				sort_order_name,					# Name of the GET attribute indicating the order in which we should sort
				
				query_name,							# Name of the GET attribute indicating a query to run
				query_type							# Type of the above query
				):
			self.paginate_page_name = paginate_page_name
			self.paginate_results_per_page_name = paginate_results_per_page_name
			
			self.sort_field_name = sort_field_name
			self.sort_order_name = sort_order_name
			
			self.query_name = query_name
			self.query_type = query_type

	client_query_vocabularies = {
		'flexigrid':	ClientQueryVocabulary('page', 'rp', 'sortname', 'sortorder', 'query', 'qtype'),		# http://www.flexigrid.info/
		}
	
	client_query_vocabulary = client_query_vocabularies['flexigrid']	# In specifying this default I'm obviously partial.
	
	paginate = True	# If we support pagination
	sort = True	# Whether we support a specified sort order
	query = True	# Whether to let the client specify a "query" attribute in the GET string
	
	_sort_order_name_map = {	# For normalizing the possible client options for specifying sort order.  This is just prepended to the order_by() argument.
		'ASC': 			'',
		'asc': 			'',
		'ASCENDING':	'',
		'ascending':	'',
		
		'DESC':			'-',
		'desc':			'-',
		'DESCENDING':	'-',
		'descending':	'-'
		}
	
	@has_model
	def read(self, request, format, *args, **kwargs):
		"""
		Get an entire collection from the model, filtered by the arguments given.
		"""
		request.GET = request.GET.copy()	# make mutable
		
		if self.paginate:
			# pop them from GET so they don't interact with field names
			try:
				page_no = request.GET[self.client_query_vocabulary.paginate_page_name]
				del request.GET[self.client_query_vocabulary.paginate_page_name]
			except KeyError:
				page_no = None
			
			try:
				results_per_page = request.GET[self.client_query_vocabulary.paginate_results_per_page_name]
				del request.GET[self.client_query_vocabulary.paginate_results_per_page_name]
			except KeyError:
				results_per_page = None
			
			# make sure they can be cast to integers
			try:
				page_no = int(page_no)
			except ValueError:
				return HttpResponseBadRequest("The page number you specified, '%s', did not resolve to an integer" % page_no)
			
			try:
				results_per_page = int(results_per_page)
			except ValueError:
				return HttpResponseBadRequest("The results per page you specified, '%s', did not resolve to an integer" % results_per_page)
			
			# we now leave to create the query before paginating
		
		if self.sort:
			# pop them from GET so they don't interact with field names
			try:
				sort_field = request.GET[self.client_query_vocabulary.sort_field_name]
				del request.GET[self.client_query_vocabulary.sort_field_name]
			except KeyError:
				sort_field = None
			
			try:
				sort_order = request.GET[self.client_query_vocabulary.sort_order_name]
				del request.GET[self.client_query_vocabulary.sort_order_name]
			except KeyError:
				sort_field = None
			
			try:
				sort_order = self._sort_order_name_map[sort_order]
			except KeyError:
				return HttpResponseBadRequest("The sort order you specified, '%s', was not recognized" % sort_order)
			
			# we now leave to create the query; then we sort it
		
		if self.query:
			try:
				query_to_execute = request.GET[self.client_query_vocabulary.query_name]
				del request.GET[self.client_query_vocabulary.query_name]
			except KeyError:
				query_to_execute = None
			
			try:
				query_type = request.GET[self.client_query_vocabulary.query_type]
				del request.GET[self.client_query_vocabulary.query_type]
			except KeyError:
				query_type = None
			
			# ... IMPLEMENT THIS.
		
		try:
			queryset = self.queryset(request).filter(**self._querydict_to_dict(request.GET))
		except FieldError, e:
			return HttpResponseBadRequest('The query you provided did not resolve to known field names: %s' % e)
		except ValueError, e:
			return HttpResponseBadRequest('One or more values did not validate: %s' % e)
		
		if self.sort and sort_field and (sort_order != None):
			# The user wants to sort the query; do it
			queryset = queryset.order_by(sort_order + sort_field)
		
		if self.paginate and page_no and results_per_page:
			# the user wants to paginate the queryset; do it
			try:
				queryset = Paginator(queryset, results_per_page).page(page_no).object_list	# Use the standard Django paginator
			except (InvalidPage, EmptyPage):
				return HttpResponseBadRequest('The pagination you specified was not valid')
		
		return queryset
	
	@has_model
	def create(self, request, format, *args, **kwargs):
		"""
		Using the data in POST, create a new member of this collection,
		and return it.
		"""
		try:
			inst = self.queryset(request).get(**request.data)
			return HttpResponseDuplicateEntry('The data you specified matches an existing entry')
		except self.model.DoesNotExist:
			if self.form:
				form = self.form(**request.data)
				if form.is_valid():
					return form.save()
				else:
					return HttpResponseBadRequest('Your data did not validate for this entry type')
			else:
				inst = self.model(**request.data)
				inst.save()
				return inst
		except self.model.MultipleObjectsReturned:
			return HttpResponseDuplicateEntry('The data you specified matches many existing entries!')
	
	@has_model
	def update(self, request, format, *args, **kwargs):
		"""
		Meaning defined as "replace the entire collection with another collection".
		"""
		pass
		"""
		The difference between this and every other REST method is that the 
		client is giving us the data of an entire COLLECTION rather than one member
		(or filtering data that looks like it could specify a member).
		
		Therefore we have to decide how data in request.POST or request.PUT should
		be structured in order to specify a collection.
		
		This is easy if the client is giving us structured JSON or XML data,
		which is capable of a sensible hierarchical structure.
		
		However, if the client wants to use standard url-encoding, we have to fall back
		on that trick used in django formsets (and elsewhere), like so:
		
		name_0=james&age_0=22&name_1=jesper&age_1=25&name_2= ...
		
		IMPLEMENT THIS.
		"""
	
	@has_model
	def delete(self, request, format, *args, **kwargs):
		"""
		Delete the entire collection (filtered by the parameters given).
		THIS IS DANGEROUS, DUDE!
		"""
		self.queryset(request).delete()
		return HttpResponseAllOK('All entries in this collection were deleted')
	
class MemberHandler(BaseHandler):
	"""
	This class implements appropriate definitions of read, create, update,
	and delete for the interpretation of the resource as a MEMBER.
	"""
	
	def _inst(self, request, kwargs):
		"""
		Get the django Model instance object specified by the client's request data.
		"""
		try:
			member_pk = kwargs[self.pkfield]
		except KeyError: # The client has not specified the primary key of this member.
			return HttpResponseBadRequest('Please specify a value for "%(identifier)s"' % { 'identifier': self.pkfield })
		else:
			try:
				return self.queryset(request).get(**{self.pkfield: member_pk})
			except ObjectDoesNotExist:
				return HttpResponseNotFound('The entry you requested could not be found')
			except MultipleObjectsReturned: # should never happen, since we're using a PK
				return HttpResponseBadRequest('The unique identification you specified returned more than one entry!')
		
	@has_model
	def read(self, request, format, *args, **kwargs):
		"""
		Get a single member, identified by primary key, and return it.
		"""
		return self._inst(request, kwargs)
	
	@has_model
	def create(self, request, format, *args, **kwargs):
		"""
		Returns 400 Bad Request.
		
		Method POST on an individual member doesn't make much sense in the abstract.
		Wikipedia says this should "[treat] the addressed member as a collection 
		in its own right and [create] a new subordinate of it".
		
		If you want this functionality then you'll have to implement it by
		overriding this function.
		"""
		return HttpResponseBadRequest('Performing a POST request on this handler has not been implemented')
	
	@has_model
	def update(self, request, format, *args, **kwargs):
		"""
		Update the addressed member of the collection or create it with the specified ID.
		"""
		inst = self._inst(request, kwargs)
		
		if not isinstance(inst, QuerySet):  # The request failed to generate a unique object.
			return inst # In this case, inst is an HttpResponse message.
		
		if self.form:
			form = self.form(request.data, instance=inst)
			if form.is_valid():
				form.save()
				return HttpResponseAllOK()
			else:
				return HttpResponseBadRequest('Your data did not validate')
		else:
			for k,v in request.data.iteritems():
				setattr( inst, k, v )
			inst.save()
			return HttpResponseAllOK()
	
	@has_model
	def delete(self, request, format, *args, **kwargs):
		"""
		Delete the addressed member of the collection.
		"""
		self._inst(request, kwargs).delete()
		return HttpResponseDeleted()

from django.db.models.fields import ( AutoField, CharField, TextField,
	FloatField, SlugField, EmailField, IntegerField, BooleanField,
	DateField, DateTimeField )

class SchemaHandler(BaseHandler):
	"""
	This class implements just one method: GET.  This returns a schema for 
	the associated model.
	"""
	_map = {
		AutoField: 'integer',
		CharField: 'string',
		TextField: 'string',
		FloatField: 'number',
		SlugField: 'string',
		EmailField: 'string',
		IntegerField: 'integer',
		BooleanField: 'boolean',
		DateField: 'string',
		DateTimeField: 'string'
		}
	
	@has_model
	def read(self, request, format, *args, **kwargs):
		"""
		In most cases, these methods can return, say, a QuerySet without any 
		concern for the way in which it is going to be formatted (JSON, XML, etc).
		This is not the case here: if the user requests JSON, we return a JSON schema;
		if XML, an XML schema, which differ in more complex ways than structure.
		"""
		if format == 'json':
			"""
			This schema format is based on the description at <http://json-schema.org/>
			"""
			schema = {
				"description": unicode(self.model._meta.verbose_name),
				"properties": {}
				}
			for field in self.model._meta.fields:
				f = {}
				cls = field.__class__
				try:
					f["type"] = self._map[cls]
				except KeyError:	# We don't know about this field type: ignore it
					pass
				else:
					# Go through different field types and populate the schema with things we can find about the field
					if cls == CharField:
						f["maxLength"] = field.max_length
				schema["properties"][field.name] = f
			return schema
		else:
			raise NotImplementedError("the requested schema format has not been implemented")

model_fields = {}
model_exclude_fields = {}

class HandlerFactoryMetaClass(type):
	def __new__(cls, name, bases, attrs):
		new_cls = type.__new__(cls, name, bases, attrs)
		if hasattr(new_cls, 'model'):
			if hasattr(new_cls, 'fields'):
				model_fields[new_cls.model] = new_cls.fields
			if hasattr(new_cls, 'exclude'):
				model_exclude_fields[new_cls.model] = new_cls.exclude
		return new_cls

class ModelHandlerFactory(object):
	__metaclass__ = HandlerFactoryMetaClass
	@classmethod
	def CollectionHandler(cls):
		class NewHandler(CollectionHandler):
			model = cls.model
			form = cls.form
			
		return NewHandler
	
	@classmethod
	def MemberHandler(cls):
		try:
			identifier = cls.identifier
		except AttributeError:
			identifier = cls.model._meta.pk.name
		
		class NewHandler(MemberHandler):
			model = cls.model
			form = cls.form
			pkfield = identifier
		return NewHandler
	
	@classmethod
	def SchemaHandler(cls):
		class NewHandler(SchemaHandler):
			model = cls.model
		return NewHandler



'''
@author: darvin
This module must be imported from django enviroment
'''


from piston.handler import AnonymousBaseHandler, BaseHandler
from piston.utils import rc
from piston.utils import validate

from forms import create_form_type
import qtdjango
from django.conf import settings
import inspect


class InfoHandler(AnonymousBaseHandler):
    allowed_methods = ('GET',)

    models = []
    server_package = None

    @staticmethod
    def create_model_info(model):
        fields = {}
        for field in model._meta.fields:
            try:
                f = {}
                f["type"]=field.__class__.__name__
                fields[field.name] = f
            except AttributeError:
                pass

        info = {"name":model.__name__,
                "fields":fields,
                }

        try:
            info["dump_order"]=model.dump_order
        except AttributeError:
            pass
        return info

    @classmethod
    def set_models(cls, models):
        for model in models:
            cls.models.append(cls.create_model_info(model))

    def read(self, request):
        #FIXME!
        project_module = __import__(getattr(settings, "ROOT_URLCONF").split(".")[0])
        project_version = project_module.__version__
        d = {
            "qtdjango_version":qtdjango.__version__,
            "server_version":project_version,
            "qtdjango_apps_setting":getattr(settings, "QTDJANGO_APPS" ),
            "models":self.models
        }
        return d



class MetaHandler(BaseHandler):
    exclude = ()
#    fields = ("machinemark",)
    allowed_methods = ('GET','POST')
    
    def has_model(self):
        return hasattr(self, 'model') or hasattr(self, 'queryset')

    def create(self, request, *args, **kwargs):
        if not self.has_model():
            return rc.NOT_IMPLEMENTED

        # get the actual data that wants to be created from the api request
        attrs = self.flatten_dict(request.POST)

        for field in self.model._meta.fields:
            if field.rel:
                # this is a ForeignKey (as far as I can tell... _meta isn't documented)
                # so we need to convert it to it's pk equivilent... there's most likely
                # a better way of doing this than hardcoding stuff...
                if field.name in attrs:
                    try:
                        attrs[field.name] =  field.rel.to.objects.get(\
                            **{field.rel.field_name: attrs[field.name]})
                    except:
                        pass


        try:
            inst = self.model.objects.get(**attrs)
            return rc.DUPLICATE_ENTRY
        except self.model.DoesNotExist:
            inst = self.model()
            inst = self.model(**attrs)
            inst.save()
            print "saved,", inst
            return inst
        except self.model.MultipleObjectsReturned:
            return rc.DUPLICATE_ENTRY


def create_handler_type(model):
    """Builds handler class from model
    @param model: Model class"""
    f = []
    for field in model._meta.fields:
        f.append(field.name)

    for methodname in dir(model):
        try:
            if getattr(model, methodname).method_as_field:
                f.append(methodname)
        except AttributeError:
            pass

    for field in model._meta.many_to_many:
        f.append(field.name)
    handler_type = type(model.__name__+"Handler", (MetaHandler,),\
                        {"model":model, "fields":f})
    valid_decor = validate(create_form_type(model))
    handler_type.create = valid_decor(handler_type.create)



    return handler_type


from _socket import socket

__author__ = 'darvin'
import restclient.restful_lib
import json

class SocketError(Exception):
    pass
class ServerNotFoundError(Exception):
    pass

class NotQtDjangoResponceError(Exception):
    pass

class AuthError(Exception):
    pass

class Connection(object):
    def __init__(self, server, url, login=None, password=None):
        self.login, self.password, self.url, self.server = login, password, url, server
        self.update_connection()

    def update_connection(self):
        self.__connection = \
            restclient.restful_lib.Connection(\
                    self.server, username=self.login, password=self.password)

    def get_resource_from_server(self, resource_name):
        """
        Gets responce from remote Django server
        @param resource_name: name of piston resource
        @rtype: dict
        """
        try:
            res = self.__connection.request_get("%s%s" % (self.url,resource_name))
        except restclient.restful_lib.httplib2.socket.error:
            raise SocketError
        except restclient.httplib2.ServerNotFoundError:
            print "serv"
            raise ServerNotFoundError

        if res["body"]=="Authorization Required":
            raise AuthError
        else:
            try:
                return json.loads(res["body"])
            except ValueError:
                raise NotQtDjangoResponceError

    def post_resource_to_server(self, resource_name, args):
        """
        Posts to remote Django server
        @param resource_name: name of piston resource
        @param body: body of post request
        @rtype: dict
        """
        res = self.__connection.request_post("%s%s" % (self.url,resource_name),args=args)
        return res

    def get_login_password(self):
        return self.login, self.password


__author__ = 'darvin'

from django import forms

class MetaForm(forms.ModelForm):
    class Meta:
        pass
    def clean(self):
        cleaned_data = self.cleaned_data

        return cleaned_data


def create_form_type(model):
    """Builds form class from model
    @param model: Model class"""
    meta_type = type("Meta", (object,), {"model":model})
    form_type = type(model.__name__+"QtDjangoForm", (MetaForm,),\
                        {"Meta":meta_type})
    return form_type





# -*- coding: utf-8 -*-
class BaseView(object):
    '''
    classdocs
    '''
    fields = ()
    widgets = {}
    model = None


    exclude_fields = ("id")
    """@cvar: Tuple of fields that must exclude from view"""

    def __init__(self, **kwargs):
        '''
        Constructor
        '''
        self.model.add_notify(self)
        if self.fields == ():
            self.fields = [x for x in self.model.get_fields()]


    def refresh(self):
        """docstring for dataChanged"""
        pass

    def clean(self):
        """Cleans temporary models of view"""
        raise NotImplementedError


    def save(self):
        """Saves models changes in view"""
        raise NotImplementedError



# -*- coding: utf-8 -*-
from PyQt4.QtGui import *
from baseview import BaseView

from widgets import *

class DetailView(QDialog, BaseView):
    fieldwidgets_dict = {TextField:TextEditWidget,
                         IdField:IdLabelWidget,
                       CharField:LineEditWidget,
                       IntegerField:SpinBoxWidget,
                       ForeignKey:ForeignKeyWidget,
                       DateTimeField:DateTimeEditWidget,
                       DateField:DateTimeEditWidget,
                       BooleanField:CheckBoxWidget,
                       }

    inline_views = ()
    """@cvar: Tuple of inline views. ((InlineViewClass, "filter_field", "Caption"),)"""

    def __init__(self, model_instance, parent=None, filter=None,**kwargs):
        QDialog.__init__(self, parent, **kwargs)
        BaseView.__init__(self, **kwargs)

        self.setWindowTitle(u"%s: редактирование" % self.model.verbose_name())

        self.model_instance = model_instance

        self.formlayout = QFormLayout()
        self._widgets={}
        self.fields = [field for field in self.fields\
                       if not getattr(self.model,field).is_read_only_in(self.model)]
        for field in self.fields:
            x = getattr(self.model,field)
            try:
                self._widgets[field] = self.widgets[field]
            except KeyError:
                try:
                    self._widgets[field] = \
                            self.fieldwidgets_dict[x.__class__](x)
                except KeyError:
                    self._widgets[field] = LabelWidget() ##FIXME!
            self.formlayout.addRow(x.get_label(), self._widgets[field])
        self.get_data_from_model()

        #then initialise inline views:
        self._inline_views = []
        for inline_view_class, inline_filter_field, inline_caption in self.inline_views:
            v = inline_view_class(filter={inline_filter_field:self.model_instance})
            #v.set_filter({inline_filter_field:self.model_instance})
            self._inline_views.append(v)
            self.formlayout.addRow(QLabel(inline_caption))
            self.formlayout.addRow(v)


        buttonBox = QDialogButtonBox(QDialogButtonBox.Ok
                                      | QDialogButtonBox.Cancel)
        Qt.QObject.connect(buttonBox, Qt.SIGNAL("accepted()"), self, Qt.SLOT("accept()"));
        Qt.QObject.connect(buttonBox, Qt.SIGNAL("rejected()"), self, Qt.SLOT("reject()"));

        v_layout = QVBoxLayout(self)
        v_layout.addLayout(self.formlayout)
        v_layout.addWidget(buttonBox)


        self.setLayout(v_layout)
        self.set_filter(filter)



    def get_data_from_model(self):
        for field in self.fields:
            self._widgets[field].setData(getattr(self.model_instance,field))

    def set_filter(self, filter):
        print "filter ", filter
        if filter is not None:
            for field, w in self._widgets.items():
                if isinstance(w, ForeignKeyWidget):
                    try:
                        if isinstance(filter[field], w.model):
                            w.setData(filter[field])
                            w.setEnabled(False)
                        else:
                            for ffieldname, ffiltervalue in filter:
                                wid_filter = {}
                                if ffieldname in w.model.get_fields():
                                    wid_filter[ffieldname] = ffiltervalue
                                w.set_filter(wid_filter)
                    except KeyError:
                        pass

    def set_data_to_model(self):
        changed = False
        for field in self.fields:
            x = self.model.get_fields()[field]
            newvalue = self._widgets[field].getData()
            if newvalue!=getattr(self.model_instance, field):
                setattr(self.model_instance, field, newvalue)
                changed = True
        if changed:
            self.model_instance.save()

        ##save inline views
        for v in self._inline_views:
            v.save()

    def validate(self):

        invalids = self.model_instance.validate()

        result = True
        for fieldname, value in invalids.items():
            if value is not None:
                field = getattr(self.model, fieldname)
                result = False
                newtext = field.get_label() +" <br> <i><font color=red>"+ value +"</color></i>"
                self.formlayout.labelForField(self._widgets[fieldname]).setText(newtext)


        return result

    def accept(self):
        self.set_data_to_model()

        if self.validate():
            QDialog.accept(self)


    def reject(self):
        for v in self._inline_views:
            v.clean()
        QDialog.reject(self)





"""
    Copyright (C) 2008 Benjamin O'Steen

    This file is part of python-fedoracommons.

    python-fedoracommons is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    python-fedoracommons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with python-fedoracommons.  If not, see <http://www.gnu.org/licenses/>.
"""

__license__ = 'GPL http://www.gnu.org/licenses/gpl.txt'
__author__ = "Benjamin O'Steen <bosteen@gmail.com>"
__version__ = '0.1'

import httplib2
import urlparse
import urllib
import base64
from base64 import encodestring

from mimeTypes import *

import mimetypes

from cStringIO import StringIO

class ConnectionError(Exception):
    def __str__(self):
        return "Connection failed"

class Connection:
    def __init__(self, base_url, username=None, password=None):
        self.base_url = base_url
        self.username = username
        m = mimeTypes()
        self.mimetypes = m.getDictionary()
        
        self.url = urlparse.urlparse(base_url)
        
        (scheme, netloc, path, query, fragment) = urlparse.urlsplit(base_url)
            
        self.scheme = scheme
        self.host = netloc
        self.path = path
        
        # Create Http class with support for Digest HTTP Authentication, if necessary
        self.h = httplib2.Http(".cache")
        self.h.follow_all_redirects = True
        if username and password:
            self.h.add_credentials(username, password)
    
    def request_get(self, resource, args = None, headers={}):
        return self.request(resource, "get", args, headers=headers)
        
    def request_delete(self, resource, args = None, headers={}):
        return self.request(resource, "delete", args, headers=headers)
        
    def request_head(self, resource, args = None, headers={}):
        return self.request(resource, "head", args, headers=headers)
        
    def request_post(self, resource, args = None, body = None, filename=None, headers={}):
        return self.request(resource, "post", args , body = body, filename=filename, headers=headers)
        
    def request_put(self, resource, args = None, body = None, filename=None, headers={}):
        return self.request(resource, "put", args , body = body, filename=filename, headers=headers)
        
    def get_content_type(self, filename):
        extension = filename.split('.')[-1]
        guessed_mimetype = self.mimetypes.get(extension, mimetypes.guess_type(filename)[0])
        return guessed_mimetype or 'application/octet-stream'
        
    def request(self, resource, method = "get", args = None, body = None, filename=None, headers={}):
        params = None
        path = resource
        headers['User-Agent'] = 'Basic Agent'
        
        BOUNDARY = u'00hoYUXOnLD5RQ8SKGYVgLLt64jejnMwtO7q8XE1'
        CRLF = u'\r\n'
        
        if filename and body:
            #fn = open(filename ,'r')
            #chunks = fn.read()
            #fn.close()
            
            # Attempt to find the Mimetype
            content_type = self.get_content_type(filename)
            headers['Content-Type']='multipart/form-data; boundary='+BOUNDARY
            encode_string = StringIO()
            encode_string.write(CRLF)
            encode_string.write(u'--' + BOUNDARY + CRLF)
            encode_string.write(u'Content-Disposition: form-data; name="file"; filename="%s"' % filename)
            encode_string.write(CRLF)
            encode_string.write(u'Content-Type: %s' % content_type + CRLF)
            encode_string.write(CRLF)
            encode_string.write(body)
            encode_string.write(CRLF)
            encode_string.write(u'--' + BOUNDARY + u'--' + CRLF)
            
            body = encode_string.getvalue()
            headers['Content-Length'] = str(len(body))
        elif body:
            if not headers.get('Content-Type', None):
                headers['Content-Type']='text/xml'
            headers['Content-Length'] = str(len(body))        
        else:
            if headers.has_key('Content-Length'):
                del headers['Content-Length']
            
            headers['Content-Type']='text/plain'
            
            if args:
                if method == "get":
                    path += u"?" + urllib.urlencode(args)
                elif method == "put" or method == "post":
                    headers['Content-Type']='application/x-www-form-urlencoded'
                    body = urllib.urlencode(args)

            
        request_path = []
        # Normalise the / in the url path
        if self.path != "/":
            if self.path.endswith('/'):
                request_path.append(self.path[:-1])
            else:
                request_path.append(self.path)
            if path.startswith('/'):
                request_path.append(path[1:])
            else:
                request_path.append(path)
        
        resp, content = self.h.request(u"%s://%s%s" % (self.scheme, self.host, u'/'.join(request_path)), method.upper(), body=body, headers=headers )
        # TODO trust the return encoding type in the decode?
        return {u'headers':resp, u'body':content.decode('UTF-8')}



from inspect import getmro
from qtdjango.models import User
#def get_registered_models(models):
#    ms =[]
#    for url, model in models.items():
#        model.resource_name = url
#        ms.append(model)
#    return ms
from qtdjango.connection import Connection


def get_resource_name_for_model(model):
    """docstring for get_resource_name_for_model"""
    app = model.__module__.split(".")[0]
    return app+r'/'+model.__name__.lower()+r's/' 


def get_all_models(app_list, from_django=False, exclude_model_names=("",)):
    """docstring for get_all_models"""
    models = []
    if from_django:
        from django.db.models import Model
        from django.contrib.auth.models import User
        model_base_class = Model
        user_class = User
    else:
        from models import Model
        from models import User
        model_base_class = Model
        user_class = User

    for app in app_list:
        m = __import__(".".join((app, "models")))
        module = getattr(m, "models")
        for name in dir(module):
            try:
                obj = getattr(module, name)
                if model_base_class in getmro(obj) and obj.__module__==app+".models" and \
                            obj.__name__ not in exclude_model_names:
                    if not from_django:
                        obj.resource_name = get_resource_name_for_model(obj)

                    models.append(obj)
            except AttributeError:
                pass

    models.append(user_class)
    user_class.resource_name=r"django/users/"
    return models

    
def test_connection(address, api_path, login, password):
    c = Connection(address, api_path, login, password)
    remote_version = c.get_resource_from_server("info")["qtdjango_version"]
    return remote_version



# -*- coding: utf-8 -*-
from qtdjango.helpers import test_connection

__author__ = 'darvin'


from qtdjango.connection import *

__author__ = 'darvin'


from PyQt4.QtCore import *
from PyQt4.QtGui import *


class BooleanEdit(QCheckBox):
    def text(self):
        return QVariant(self.checkState()).toString()
    def setText(self, text):
        self.setChecked(QVariant(text).toBool())


class SettingsDialog(QDialog):


    widgets_table = [
#            (name, caption, widget object, default value),
            ("address", u"Адрес сервера", QLineEdit, "http://127.0.0.1:8000"),
            ("api_path", u"Путь к api сервера", QLineEdit, "/api/"),
            ("server_package", u"Название пакета сервера", QLineEdit, "cryotec_server"),
            ("login", u"Ваш логин", QLineEdit, ""),
            ("password", u"Ваш пароль", QLineEdit, ""),
            ("open_links_in_external_browser", \
             u"Открывать ссылки из окна информации во внешнем браузере", BooleanEdit, True),

        ]
    def __init__(self, parent=None, error_message=None, models_manager=None):
        super(SettingsDialog, self).__init__(parent)
        self.setModal(True)
        self.formlayout = QFormLayout()
        self.models_manager = models_manager
        self.settings = QSettings()
        self.message_widget = QLabel()
        self.__widgets = []
        for name, caption, widget_class, default in self.widgets_table:
            self.__widgets.append((name, caption, widget_class(), default))

        for name, caption, widget, default in self.__widgets:
            self.formlayout.addRow(caption, widget)
            widget.setText(self.settings.value(name, default).toString())

        self.formlayout.addRow(self.message_widget)

        if error_message is not None:
            self.message(**error_message)

        buttonBox = QDialogButtonBox(QDialogButtonBox.Save\
            | QDialogButtonBox.Cancel |QDialogButtonBox.RestoreDefaults)

        testButton = QPushButton(u"Тестировать соединение")
        buttonBox.addButton(testButton, QDialogButtonBox.ActionRole)
        testButton.clicked.connect(self.test)
        resetButton = QPushButton(u"Сбросить настройки")
        buttonBox.addButton(resetButton, QDialogButtonBox.DestructiveRole)
        resetButton.clicked.connect(self.reset)
        buttonBox.accepted.connect(self.accept)
        buttonBox.rejected.connect(self.reject)
        buttonBox.button(QDialogButtonBox.RestoreDefaults).clicked.connect(self.restore)
        self.formlayout.addRow(buttonBox)
        self.setLayout(self.formlayout)

    def accept(self):
        if self.test():
            for name, caption, widget, default in self.__widgets:
                self.settings.setValue(name, widget.text())

            self.models_manager.set_connection_params(\
                    self.get_value("address"), \
                    self.get_value("api_path"), \
                    self.get_value("login"),\
                    self.get_value("password"))
            QDialog.accept(self)



    def restore(self):
        for name, caption, widget, default in self.__widgets:
            widget.setText(default)

    def message(self, text, error=False, works=False, fields=[]):
        self.message_widget.setText(text)
        if error:
            color = "red"
        elif works:
            color = "green"
        else:
            color = "black"

        css = "QLabel { color : %s; }" % color
        self.message_widget.setStyleSheet(css)
        for name, caption, widget, default in self.__widgets:
            self.formlayout.labelForField(widget).setStyleSheet("")
            if name in fields:
                self.formlayout.labelForField(widget).setStyleSheet(css)

    def get_value(self, name):
        return unicode(self.settings.value(name).toString())

    def test(self):
        s = {}
        for name, caption, widget, default in self.__widgets:
            s[name] = unicode(widget.text())


        try:
            remote_version = test_connection(s["address"],s["api_path"],s["login"],s["password"])

            import qtdjango
            if qtdjango.__version__==remote_version:
                self.message(text=u"Удаленный сервер настроен правильно!", works=True)
                return True
            elif remote_version is not None:
                self.message(u"Версия системы на удаленном сервере отличается от\
версии системы на клиенте")
                return True

        except SocketError:
            self.message(text=u"Ошибка при подключении к удаленному серверу", error=True, fields=\
                            ("address",))
        except ServerNotFoundError:
            self.message(text=u"Удаленный сервер недоступен", error=True, fields=\
                            ("address",))
        except NotQtDjangoResponceError:
            self.message(text=u"Не правильно настроен путь на удаленном сервере или \
удаленный сервер не является сервером системы", error=True, fields=\
                            ("address","api_path"))

        except AuthError:
            self.message(text=u"Неверное имя пользователя или пароль", error=True, fields=\
                            ("login","password"))


        return False




    def reset(self):
        print "reset"



# -*- coding: utf-8 -*-


'''
@author: darvin
'''
from qtmodels import ListModel
from models import *
from PyQt4.QtGui import *
from PyQt4 import QtCore, Qt
from PyQt4.QtCore import QDateTime


class Widget(object):
    def __init__(self, field):
        self.field = field

        
    def getData(self):
        raise NotImplementedError
    
    def setData(self, data):
        raise NotImplementedError


class LabelWidget(QLabel, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QLabel.__init__(self)
        
    def getData(self):
        return unicode(self.text())
    
    def setData(self, data):
        if data is not None:
            self.setText(str(data))

class ForeignKeyWidget(QComboBox, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QComboBox.__init__(self)
        self.model = self.field.model


    def set_filter(self, filter):
        self.qtmodel.set_filter(filter)

    def getData(self):
        try:
            return self.qtmodel.get_model_instance_by_int_index(\
                self.currentIndex())
        except IndexError:
            return None

    def setData(self, data):
        self.qtmodel = ListModel(parent=self, filter=None,\
                                model=self.model, blank_variant=self.field.blank)
        self.setModel(self.qtmodel)
        if data is not None:
            self.setCurrentIndex(self.qtmodel.get_index_of_model(data))



class SpinBoxWidget(QSpinBox, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QLabel.__init__(self)
        self.setMinimum(0)
        self.setMaximum(999999)

    def getData(self):
        return self.value()
    
    def setData(self, data):
        if data is not None:
            self.setValue = data

class DateTimeEditWidget(QDateTimeEdit, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QDateTimeEdit.__init__(self)

    def getData(self):
        return self.dateTime().toPyDateTime()

    def setData(self, data):
        if data is not None:
            self.setDateTime(data)
        else:
            self.setDateTime(QDateTime.currentDateTime())

class CheckBoxWidget(QCheckBox, Widget):
    def __init__(self, field):
       Widget.__init__(self, field)
       QCheckBox.__init__(self)

    def getData(self):
        return self.isChecked()
    def setData(self, data):
        if data is None:
            data = False
        self.stateChanged.emit(data)

class ComboBoxWidget(QComboBox, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QComboBox.__init__(self)
        
    def getData(self):
        return self.value
    
    def setData(self, data):
        if data is not None:
            self.value = data


class TextEditWidget(QTextEdit, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QTextEdit.__init__(self)
        self.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))

    def getData(self):
        return unicode(self.toPlainText())
    
    def setData(self, data):
        if data is not None:
            self.setPlainText(QtCore.QString(data))


class LineEditWidget(QLineEdit, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QLineEdit.__init__(self)

    def getData(self):
        return unicode(self.text())
    
    def setData(self, data):
        if data is not None:
            self.setText(QtCore.QString(data))


class IdLabelWidget(QLabel, Widget):
    def __init__(self, field):
        Widget.__init__(self, field)
        QLabel.__init__(self)

    def getData(self):
        res = self.text().toInt()
        if res[1]:
            return res[0]
        else:
            return None
    
    def setData(self, data):
        if data is not None:
            self.setText(str(data))





"""
    Copyright (C) 2008 Benjamin O'Steen

    This file is part of python-fedoracommons.

    python-fedoracommons is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    python-fedoracommons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with python-fedoracommons.  If not, see <http://www.gnu.org/licenses/>.
"""

__license__ = 'GPL http://www.gnu.org/licenses/gpl.txt'
__author__ = "Benjamin O'Steen <bosteen@gmail.com>"
__version__ = '0.1'

class mimeTypes(object):
    def getDictionary(self):
        mimetype_to_extension = {}
        extension_to_mimetype = {}
        mimetype_to_extension['text/plain'] = 'txt'
        mimetype_to_extension['text/xml'] = 'xml'
        mimetype_to_extension['text/css'] = 'css'
        mimetype_to_extension['text/javascript'] = 'js'
        mimetype_to_extension['text/rtf'] = 'rtf'
        mimetype_to_extension['text/calendar'] = 'ics'
        mimetype_to_extension['application/msword'] = 'doc'
        mimetype_to_extension['application/msexcel'] = 'xls'
        mimetype_to_extension['application/x-msword'] = 'doc'
        mimetype_to_extension['application/vnd.ms-excel'] = 'xls'
        mimetype_to_extension['application/vnd.ms-powerpoint'] = 'ppt'
        mimetype_to_extension['application/pdf'] = 'pdf'
        mimetype_to_extension['text/comma-separated-values'] = 'csv'
        
        
        mimetype_to_extension['image/jpeg'] = 'jpg'
        mimetype_to_extension['image/gif'] = 'gif'
        mimetype_to_extension['image/jpg'] = 'jpg'
        mimetype_to_extension['image/tiff'] = 'tiff'
        mimetype_to_extension['image/png'] = 'png'
        
        # And hacky reverse lookups
        for mimetype in mimetype_to_extension:
            extension_to_mimetype[mimetype_to_extension[mimetype]] = mimetype
        
        mimetype_extension_mapping = {}
        mimetype_extension_mapping.update(mimetype_to_extension)
        mimetype_extension_mapping.update(extension_to_mimetype)
        
        return mimetype_extension_mapping



# -*- coding: utf-8 -*-
from PyQt4.QtGui import QTreeWidget, QTreeWidgetItem, QDesktopServices
from PyQt4 import QtCore
from qtdjango.models import Model
from PyQt4.QtWebKit import QWebView, QWebPage
#noinspection PyUnresolvedReferences
import PyQt4.QtNetwork
from PyQt4.QtCore import QSettings

__author__ = 'darvin'

class MultiModelView(object):
    models = None
    def __init__(self):
        for model in self.models:
            model.add_notify(self)



    def refresh(self):
        """Refreshs model"""
        pass





class ModelTreeWidgetItem(QTreeWidgetItem):
    def __init__(self,  parent, model_instance):
        super(ModelTreeWidgetItem, self).__init__(parent)
        self.model_instance = model_instance
        self.setText(0, unicode(self.model_instance))


class MultiModelTreeView(QTreeWidget, MultiModelView):
    tree_structure = None
    """@cvar: Tuple, structure of tree (("fieldname", ModelClass))"""
    item_class = ModelTreeWidgetItem
    """@cvar: Class of items"""
    modelSelectionChanged = QtCore.pyqtSignal([Model])
    modelSelectionCleared = QtCore.pyqtSignal()


    def __process_node(self, node=None, level=0, parenttreeitem=None):
        if node is None:
            subnodes = self.tree_structure[level][1].all()
        else:
            subnodes = self.tree_structure[level][1].filter(**{self.tree_structure[level-1][0]:node})

        result = []
        for subnode in subnodes:
            result.append(subnode)
            treeitem = self.item_class(parenttreeitem, model_instance=subnode)

            if level<len(self.tree_structure)-1:
                result.append(self.__process_node(subnode, level+1, treeitem))
        return result

    def __init__(self, *args, **kwargs):
        QTreeWidget.__init__(self, *args, **kwargs)
        MultiModelView.__init__(self, *args, **kwargs)
        self.header().hide()
        self.refresh()
        self.currentItemChanged.connect(self.currentItemChange)


    def refresh(self):
        self.clear()
        self.__process_node(parenttreeitem=self)

    @QtCore.pyqtSlot("QTreeWidgetItem*", "QTreeWidgetItem*")
    def currentItemChange(self, current, previous):
        try:
            self.modelSelectionChanged.emit(current.model_instance)
        except AttributeError:
            self.modelSelectionCleared.emit()



class ModelInfoView(QWebView, MultiModelView):
    def __init__(self, *args, **kwargs):

        QWebView.__init__(self, *args, **kwargs)
        MultiModelView.__init__(self, *args, **kwargs)
        self.setHtml("")
#        if self.open_link_in_external_browser():
        self.page().setLinkDelegationPolicy(QWebPage.DelegateAllLinks)
        self.linkClicked.connect(self.link_clicked)

    def link_clicked(self, link):
        print self.open_link_in_external_browser()
        if self.open_link_in_external_browser():
            QDesktopServices.openUrl(link)
        else:
            self.load(link)



    def refresh(self):
        try:
            self.modelChanged(self.model_instance)
        except AttributeError:
            pass

    def open_link_in_external_browser(self):
        return QSettings().value("open_links_in_external_browser", True).toBool()

    @QtCore.pyqtSlot(Model)
    def modelChanged(self, model):
        self.model_instance = model
        header1 = model.__class__.verbose_name()
        header2 = "" #unicode(model)
        fields = model.__class__.get_fields()
        field_text_values = {}
        for fieldname, field in fields.items():
            if not fieldname in ("user", "id", "extra_to_html"):
                field_text_values[fieldname] = []
                field_text_values[fieldname].append(field.verbose_name)
                field_text_values[fieldname].append(field.to_text(getattr(model, fieldname)))


        html = u""
        html += u"<h1>%s</h1><h2>%s</h2>" %(header1,header2)
        html += u"<br>".join([u"<b>%s:</b> <i>%s</i>"%(x[0], x[1]) for x in field_text_values.values()])

        html += "<br>" + model.extra_to_html()

        self.setHtml(html)

    @QtCore.pyqtSlot()
    def modelCleared(self):
        self.setHtml("")






from restful_lib import Connection, ConnectionError

# from gae_restful_lib import GAE_Connection

TINYURL_ENDPOINT = "http://tinyurl.com/api-create.php"
TINYURL_PARAM = "url"

class Tinyurl(object):
    def __init__(self):
        self._conn = Connection(TINYURL_ENDPOINT)
        # TODO test availability
        self.active = True
    
    def get(self, url):
        # Handcraft the ?url=XXX line as Tinyurl doesn't understand urlencoded
        # params - at least, when I try it anyway...
        response = self._conn.request_get("?%s=%s" % (TINYURL_PARAM, url))
        http_status = response['headers'].get('status')
        if http_status == "200":
            return response.get('body').encode('UTF-8')
        else:
            raise ConnectionError



import urllib2

from urllib2 import urlparse

from restful_lib import Connection

#from gae_restful_lib import GAE_Connection

from datetime import datetime

from StringIO import StringIO

from xml.etree import ElementTree as ET

SPARQL_ENDPOINT = "/services/sparql"
META_ENDPOINT = "/meta"
CONTENT_ENDPOINT = "/items"
JOB_REQUESTS = "/jobs"
SNAPSHOTS = "/snapshots"
SNAPSHOT_TEMPLATE = "/snapshots/%s"

RESET_STORE_TEMPLATE = u"""<rdf:RDF
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
    xmlns:bf="http://schemas.talis.com/2006/bigfoot/configuration#" > 
  <bf:JobRequest>
    <rdfs:label>%s</rdfs:label>
    <bf:jobType rdf:resource="http://schemas.talis.com/2006/bigfoot/configuration#ResetDataJob"/>
    <bf:startTime>%sZ</bf:startTime>
  </bf:JobRequest>
 </rdf:RDF>"""

SNAPSHOT_STORE_TEMPLATE = """<rdf:RDF
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
    xmlns:bf="http://schemas.talis.com/2006/bigfoot/configuration#" > 
  <bf:JobRequest>
    <rdfs:label>%s</rdfs:label>
    <bf:jobType rdf:resource="http://schemas.talis.com/2006/bigfoot/configuration#SnapshotJob"/>
    <bf:startTime>%sZ</bf:startTime>
  </bf:JobRequest>
 </rdf:RDF>"""

SNAPSHOT_RESTORE_TEMPLATE = """<rdf:RDF
    xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
    xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#"
    xmlns:bf="http://schemas.talis.com/2006/bigfoot/configuration#" > 
  <bf:JobRequest>
    <rdfs:label>%s</rdfs:label>
    <bf:jobType rdf:resource="http://schemas.talis.com/2006/bigfoot/configuration#RestoreJob"/>
    <bf:snapshotUri rdf:resource="%s" />
    <bf:startTime>%sZ</bf:startTime>
  </bf:JobRequest>
 </rdf:RDF>"""

class RDFFormatException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)

class Store():
    def __init__(self, base_store_url, username=None, password=None):
        """ Base URL for the store should be pretty self-explanatory. E.g. something like
            "http://api.talis.com/stores/store_name"
            Only needs to enter the username/password if this class is going to tinker
            with things."""
        if base_store_url.endswith('/'):
            base_store_url = base_store_url[:-1]

        self.base_store_url = base_store_url
        # Split the given URL
        if base_store_url:
            self.conn = Connection(base_store_url, username=username, password=password)

    def does_snapshot_exist(self, snapshot_filename):
        # Test to see if snapshot exists:
        snapshot_path = SNAPSHOT_TEMPLATE % snapshot_filename
        
        response = self.conn.request(snapshot_path, method = "HEAD")
        
        if response.get('headers') and response.get('headers').get('status'):
            status = response.get('headers').get('status')
        
            if status in ['200', '204']:
                return True
            elif status.startswith('4'):
                return False
            # else: raise Error?

        return False

    def schedule_reset_data(self, label, at_time=None):
        """Will request that the store is emptied, and label the request. 
           If a time is given as an ISO8601 formatted string, this will be 
           the scheduled time for the snapshot. Otherwise, it will use the current time."""
        if not at_time:
            at_time=datetime.utcnow().isoformat().split('.')[0]
        
        snapshot_request = RESET_STORE_TEMPLATE % (label, at_time)
        
        return self.conn.request_post(JOB_REQUESTS, body = snapshot_request, headers={'Content-Type':'application/rdf+xml'})

    def schedule_snapshot_data(self, label, at_time=None):
        """Will request a snapshot be made of the store. 
           If a time is given as an ISO8601 formatted string, this will be 
           the scheduled time for the snapshot. Otherwise, it will use the current time."""
        if not at_time:
            at_time=datetime.utcnow().isoformat().split('.')[0]
        
        snapshot_request = SNAPSHOT_STORE_TEMPLATE % (label, at_time)
        
        return self.conn.request_post(JOB_REQUESTS, body = snapshot_request, headers={'Content-Type':'application/rdf+xml'})
        
    def schedule_snapshot_restore(self, label, snapshot_filename, at_time=None):
        """Will request that the store is restored from a snapshot. If a time is given as
           an ISO8601 formatted string, this will be the scheduled time for
           the recovery. Otherwise, it will use the current time."""
        if not at_time:
            at_time=datetime.utcnow().isoformat().split('.')[0]
        
        # Test to see if snapshot exists:
        snapshot_path = SNAPSHOT_TEMPLATE % snapshot_filename
        
        if self.does_snapshot_exist(snapshot_filename):
            snapshot_uri = "%s%s" % (self.base_store_url, snapshot_path)
            snapshot_request = SNAPSHOT_RESTORE_TEMPLATE % (label, snapshot_uri, at_time)    
            return self.conn.request_post(JOB_REQUESTS, body = snapshot_request, headers={'Content-Type':'application/rdf+xml'})
            
    def submit_rdfxml(self, rdf_text):
        """Puts the given RDF/XML into the Talis Store"""
        return self._put_rdf(rdf_text, mimetype="application/rdf+xml")
        
    def _put_rdf(self, rdf_text, mimetype="application/rdf+xml"):
        """Placeholder for allowing other serialisation types to be put into a
           Talis store, whether the conversion takes place here, or if the Talis
           store starts to accept other formats."""
        if rdf_text:
            request_headers = {}
            if mimetype not in ['application/rdf+xml']:
                raise RDFFormatException("%s is not an allowed RDF serialisation format" % mimetype)
            request_headers['Content-Type'] = mimetype
            return self.conn.request_post(META_ENDPOINT, body=rdf_text, headers=request_headers)        
                 
    def _query_sparql_service(self, query, args={}):
        """Low-level SPARQL query - returns the message and response headers from the server.
           You may be looking for Store.sparql instead of this."""
        passed_args = {'query':query}
        passed_args.update(args)
        return self.conn.request_get(SPARQL_ENDPOINT, args=passed_args, headers={'Content-type':'application/x-www-form-urlencoded'})
        
    def _query_search_service(self, query, args={}):
        """Low-level content box query - returns the message and response headers from the server.
           You may be looking for Store.search instead of this."""
           
        passed_args = {'query':query}
        passed_args.update(args)
        
        return self.conn.request_get(CONTENT_ENDPOINT, args=passed_args, headers={'Content-type':'application/x-www-form-urlencoded'} )
        
    def _list_snapshots(self, passed_args={}):
        return self.conn.request_get(SNAPSHOTS, args=passed_args, headers={}) 
        
##############################################################################
# Convenience Functions
##############################################################################

    def submit_rdfxml_from_url(self, url_to_file, headers={"Accept":"application/rdf+xml"}):
        """Convenience method - downloads the file from a given url, and then pushes that
           into the meta store. Currently, it doesn't put it through a parse-> reserialise
           step, so that it could handle more than rdf/xml on the way it but it is a
           future possibility."""
        import_rdf_connection = Connection(url_to_file)
        response = import_rdf_connection.request_get("", headers=headers)
        
        if response.get('headers') and response.get('headers').get('status') in ['200', '204']:
            request_headers = {}
            
            # Lowercase all response header fields, to make matching easier. 
            # According to HTTP spec, they should be case-insensitive
            response_headers = response['headers']
            for header in response_headers:
                response_headers[header.lower()] = response_headers[header]
                
            # Set the body content
            body = response.get('body').encode('UTF-8')
            
            # Get the response mimetype
            rdf_type = response_headers.get('content-type', None)
            
            return self._put_rdf(body, mimetype=rdf_type)
            
    def sparql(self, query, args={}):
        """Performs a SPARQL query and simply returns the body of the response if successful
           - if there is an issue, such as a code 404 or 500, this method will return False. 
           
           Use the _query_sparql_service method to get hold of
           the complete response in this case."""
        response = self._query_sparql_service(query, args)
        headers = response.get('headers')
        
        status = headers.get('status', headers.get('Status'))
        
        if status in ['200', 200, '204', 204]:
            return response.get('body').encode('UTF-8')
        else:
            return False

    def search(self, query, args={}):
        """Performs a search query and simply returns the body of the response if successful
           - if there is an issue, such as a code 404 or 500, this method will return False. 
           
           Use the _query_search_service method to get hold of
           the complete response in this case."""
        response = self._query_search_service(query, args)
        headers = response.get('headers')
        
        status = headers.get('status', headers.get('Status'))
        
        if status in ['200', 200, '204', 204]:
            parsed_atom = Atom_Search_Results(response.get('body').encode('UTF-8'))
            return parsed_atom.get_item_list()
        else:
            return False
            
class Item():
    def __init__(self):
        self.title = None
        self.link = None

class Atom_Search_Results():
    def __init__(self, atom_text):
        self.load_atom_search(atom_text)
    
    def load_atom_search(self, atom_text):
        self.atom = ET.fromstring(atom_text)
    
    def get_item_list(self):
        if self.atom:
            items = []
            for item in self.atom.findall('{http://purl.org/rss/1.0/}item'):
                item_fields = Item()
                item_fields.title = item.find('{http://purl.org/rss/1.0/}title').text
                item_fields.link = item.find('{http://purl.org/rss/1.0/}link').text
                items.append(item_fields)
                
            return items
            
class GAE_Store(Store):
    def __init__(self, base_store_url, username=None, password=None):
        """ Base URL for the store should be pretty self-explanatory. E.g. something like
            "http://api.talis.com/stores/store_name"
            The username and password will not do anything, until the Google app engine's
            fetch library handles authentication, if ever."""
        if base_store_url.endswith('/'):
            base_store_url = base_store_url[:-1]

        self.base_store_url = base_store_url
        # Split the given URL
        if base_store_url:
            self.conn = GAE_Connection(base_store_url, username, password)
        



from django.conf import settings

from qtdjango.helpers import get_resource_name_for_model, get_all_models
from piston.resource import Resource
from piston.authentication import HttpBasicAuthentication

from handlers import create_handler_type, InfoHandler
from django.conf.urls.defaults import url, patterns



def create_resource(model):
    """Builds resource from model class
    @param model: Model class"""

    return Resource(create_handler_type(model), HttpBasicAuthentication())

def get_url_pattens(app_list):
    """Gets app list returns urls patterns"""
    models = get_all_models(app_list, from_django=True)
    urlpatterns = []
    models_list = []
    for model in models:
        urlpatterns.append(url(r"^"+get_resource_name_for_model(model),\
            create_resource(model)))

    InfoHandler.set_models(models)
    urlinfo = url(r"^info", Resource(InfoHandler, HttpBasicAuthentication()))
    urlpatterns.append(urlinfo)


    return patterns("", * urlpatterns)



urlpatterns = get_url_pattens(getattr(settings, "QTDJANGO_APPS" ))



# -*- coding: utf-8 -*-
from PyQt4.QtGui import *
from PyQt4 import QtCore, Qt
from baseview import BaseView
from qtmodels import *
from models import Model

class UndetailView(BaseView):

    sort_by = None
    """@cvar sort by column"""

    def __init__(self, filter=None):
        BaseView.__init__(self)
        self.filter = filter

    def set_filter(self, filter):
        """Sets filter. Filter is dict"""
        self.filter = filter

    def clean(self):
        """Cleans temporary models of view"""
        pass



class UndetailWithButtonsView(QFrame, UndetailView):
    """Proxy class for UndetailView with buttons"""
    edit_dumped = True
    """@cvar: Allows edit and delete entries on server"""
    viewclass = None
    """@cvar: Class of detail view for editing entires"""
    buttons = {
               "delete": (True, u"Удалить"),
               "edit":(True,u"Редактировать"),
               "new":(False,u"Добавить"),
              }
    """@cvar: Buttons
    'name': (undumped only, 'caption'),"""

    edit_filtered_only = False
    """@cvar: allow edit only if view is filtered"""

    def __init__(self, filter=None, *args, **kwargs):
        """docstring for __init__"""
        QFrame.__init__(self)
        self.view = self.viewclass()
        self.layout = QVBoxLayout()
        self.setLayout(self.layout)
        self.layout.addWidget(self.view)
        self.buttonlayout = QHBoxLayout()
        self.layout.addLayout(self.buttonlayout)
        self._buttons = {}
        for button, (dumped_only, buttonname) in self.buttons.items():
            b = QPushButton(buttonname)
            self._buttons[button] = b
            self.buttonlayout.addWidget(b)
            #FIXME
            #l = lambda x: getattr(self.view,"model_"+button)()
            #print l
            #b.clicked.connect(l)
        self._buttons["edit"].clicked.connect(lambda x: self.view.model_edit())
        self._buttons["delete"].clicked.connect(lambda x: self.view.model_delete())
        self._buttons["new"].clicked.connect(lambda x: self.view.model_new())
        self.view.modelSelectionChanged.connect(self.modelSelectionChange)

        self.refresh_buttons_state()
        self.set_filter(filter)

    def get_buttons_state(self, model_selected=None):
        try:
            model_selected_undumped = not model_selected.is_dumped()
        except AttributeError:
            model_selected_undumped = True

        state = {}
        for button, (undumped_only, buttonname) in self.buttons.items():
            if self.edit_filtered_only and self.view.filter is None:
                state[button] = False
            else:
                if not self.edit_dumped and undumped_only and \
                   (not model_selected_undumped or model_selected is None):
                    state[button] = False
                else:
                    state[button] = True

        return state


    def refresh_buttons_state(self, model=None):
        """Disables buttons for edit, if not edit allowed"""
        current_state = self.get_buttons_state(model)
        for button, (undumped_only, buttonname) in self.buttons.items():
            self._buttons[button].setDisabled(not current_state[button])

    @QtCore.pyqtSlot(Model)
    def modelSelectionChange(self, model):
        self.refresh_buttons_state(model)

    def set_filter(self, filter):
        """Sets filter. Filter is dict"""
        self.view.set_filter(filter)
        self.refresh_buttons_state()

    def save(self):
        self.view.save()





class AbstactQtModelUndetailView(UndetailView, QAbstractItemView):
    def __init__(self, filter=None):
        QAbstractItemView.__init__(self)
        UndetailView.__init__(self, filter)
        self.qtmodel = self._qt_model_class(self.model, self.filter, self.fields)
        self.sortmodelproxy = QSortFilterProxyModel(parent=self)
        self.sortmodelproxy.setSourceModel(self.qtmodel)
        self.setModel(self.sortmodelproxy)
    def set_filter(self, filter):
        UndetailView.set_filter(self, filter)
        self.qtmodel.set_filter(self.filter)

    def create_detail_view(self, model_instance, filter=None):

        #t = type(self.model.__name__.upper() + "DetailView",\
                 #(DetailView,), {"model":model_to_edit})
        return self.detail_view(parent=self, model_instance=model_instance,\
                                filter=filter)

    @QtCore.pyqtSlot()
    def currentChanged (self, mi1, mi2):
        self.modelSelectionChanged.emit( self.qtmodel.data(\
                self.sortmodelproxy.mapToSource(mi1), QtCore.Qt.UserRole))


    def model_edit (self):
        dv = self.create_detail_view(\
                    self.qtmodel.data(\
                            self.sortmodelproxy.mapToSource(self.currentIndex()),\
                            QtCore.Qt.UserRole))
        dv.show()

    def model_delete (self):
        m = self.qtmodel.data(\
                            self.sortmodelproxy.mapToSource(self.currentIndex()),\
                            QtCore.Qt.UserRole)
        m.delete()

    def create_model_instance(self):
        return self.model.new()

    def model_new (self):
        new_model = self.create_model_instance()
        dv = self.create_detail_view(model_instance=new_model, filter=self.filter)
        result = dv.exec_()
        if result==QDialog.Accepted:
            pass
        elif result==QDialog.Rejected:
            new_model.delete()

    def refresh(self):
        self.qtmodel.reset()

    def save(self):
        """Does nothing, all changes saves instantly"""
        pass

class ListView(QListView, AbstactQtModelUndetailView):
    _qt_model_class = ListModel
    modelSelectionChanged = QtCore.pyqtSignal([Model]) ##dont work when in
                                                        #father class
    def __init__(self, filter=None):
        QListView.__init__(self)
        AbstactQtModelUndetailView.__init__(self, filter)




class TableView(QTableView, AbstactQtModelUndetailView):
    _qt_model_class = TableModel
    modelSelectionChanged = QtCore.pyqtSignal([Model]) ##dont work when in
                                                        #father class


    def __init__(self, filter=None):
        QTableView.__init__(self)
        AbstactQtModelUndetailView.__init__(self, filter)
        self.setSortingEnabled(True)
#        self.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
        self.verticalHeader().hide()
        if self.sort_by:
            if self.sort_by[0]=="-":
                sort_order = QtCore.Qt.DescendingOrder
                sort_by = self.sort_by[1:]
            else:
                sort_order = QtCore.Qt.AscendingOrder
                sort_by = self.sort_by

            self.sortByColumn(self.fields.index(sort_by), sort_order)






class TreeView(QTreeView, AbstactQtModelUndetailView):
    _qt_model_class = TreeModel
    modelSelectionChanged = QtCore.pyqtSignal([Model]) ##dont work when in
                                                        #father class

    modelSelectionCleared = QtCore.pyqtSignal()
    tree_structure = {}
    """@cvar: Dict of tree structure {fk field name: model class}"""

    def __init__(self, filter=None):
        QTreeView.__init__(self)
        AbstactQtModelUndetailView.__init__(self, filter)
        self.qtmodel.set_tree_structure(self.tree_structure)




# -*- coding: UTF-8 -*-

'''
@author: darvin
'''





from inspect import getmro
import json


def istype(obj, typename):
    try:
        return typename in [clsobj.__name__ for clsobj in getmro(obj.__class__)]
    except AttributeError:
        return False

class Field(object):
    widget = None
    always_read_only = False
    def __init__(self, verbose_name=None, *args, **kwargs):
        try:
            self.verbose_name = verbose_name
        except IndexError:
            self.verbose_name = ""
        self.__read_only =[]
        try:
            self.null = kwargs["null"]
        except KeyError:
            self.null = False
        try:
            self.blank = kwargs["blank"]
        except KeyError:
            self.blank = False


    def from_raw(self, data):
        return data

    def set_read_only_in(self, model_class):
        self.__read_only.append(model_class)

    def is_read_only_in(self, model_class):
        return (model_class in self.__read_only) or self.always_read_only

    def to_raw(self, data):
        return data

    def to_text(self, data):
        if data is None:
            return u"<пусто>"
        else:
            return unicode(data)

    def validate(self, data, model_instance):
        if data is None or data=="":
            if self.blank or self.null or self.is_read_only_in(model_instance.__class__):
                return None
            else:
                return u"Поле не может быть пустым"
        else:
            return None


    def get_blank(self):
        return None

    def get_label(self):
        try:
            return self.verbose_name
        except AttributeError:
            return "verbose name not defined"


class IdField(Field):
    always_read_only = True
    def __init__(self, *args, **kwargs):
        super(IdField, self).__init__(*args, **kwargs)
        self.read_only = True

    def load(self, data):
        return int(data)

class TextField(Field):
    pass

from datetime import datetime
class DateField(Field):
    def from_raw(self, data):
        try:
            return datetime.strptime(data, "%Y-%m-%d")
        except ValueError:
            return datetime.strptime(data, "%Y-%m-%d %H:%M:%S")
    def to_raw(self, data):
        return data.strftime("%Y-%m-%d")

    def to_text(self, data):
        try:
            return data.strftime("%d.%m.%Y %H:%M")
        except AttributeError:
            return super(DateField, self).to_text(data)


class DateTimeField(DateField):
    pass

class CharField(Field):
    pass


class URLField(CharField):
    def to_text(self, data):
        return u'<a href="%s">%s</a>' %(data, data)


class BooleanField(Field):
    pass
    def to_raw(self, data):
        return data

    def to_text(self, data):
        if data:
            return u"да"
        else:
            return u"нет"

class EmailField(CharField):
    pass
    def to_raw(self, data):
        raise NotImplementedError


class FileField(CharField):
    pass
    def to_raw(self, data):
        raise NotImplementedError

class IntegerField(Field):
    def from_raw(self, data):
        if data is not None:
            return int(data)
        else:
            return None

    def to_raw(self, data):
        if data is not None:
            return int(data)
        else:
            return None


class PositiveIntegerField(IntegerField):
    pass

class RelField(Field):
    pass

class ForeignKey(RelField):

    def from_raw(self, data):
        if data is not None:
            if self.model.loaded:
                return self.model.get(data["id"])
            else:
                return data
        else:
            return None
#    def to_text(self, data):
#        return unicode(data)

    def to_raw(self, data):
        try:
            return data.id
        except AttributeError:
            return None



    def __init__(self, model, verbose_name=None, *args, **kwargs):
        super(ForeignKey, self).__init__(verbose_name=verbose_name, *args, **kwargs)
        self.model = model




class ManyToManyField(ForeignKey):
    def from_raw(self, data):

        if data is not None:
            if self.model.loaded:
                try:
                    res = []
                    for item in data:
                        res.append(self.model.get(item["id"]))
                    return res
                except TypeError:
                    return data
            else:
                return data
        else:
            return None
    def to_text(self, data):
            return unicode(data)

    def to_raw(self, data):
        raise NotImplementedError



class ResourceNameError(Exception):
    pass


class Model(object):
    '''
    classdocs
    '''
    resource_name = None
    """@cvar: This is resource name of model"""

    loaded = False
    """@cvar: This is flag, is model loaded"""

    __models_manager = None
    """@cvar: This is ModelManager object, throuth that model
    works with remote Django server"""

    objects = []
    """@cvar: List of Model instances of this model"""

    views = []
    """@cvar: List of views, connected to this model"""

    exclude_methods = ("save",)
    """@cvar: List of methods of models, that must be subtitude by Model class methods"""

    read_only_fields = []
    """@cvar: List of fields to read-only"""

    auto_user_fields = []


    dump_order = 0
    """@cvar: Order to dump"""

    @classmethod
    def load(cls):
        """Loads all model objects from server via ModelsManager.
        Sets initial [] values for objects and views"""
        if not cls.loaded:
            if cls.resource_name is None:
                raise ResourceNameError
            cls.id = IdField("Id")
            raw = cls.__models_manager.get_resource_from_server(cls.resource_name)


            for x in raw:
                o = cls(**x)
                o.__dumped=True
                o.version = cls.version
                cls.objects.append(o)
            cls.loaded = True
    #FIXME
    @classmethod
    def refresh_foreing_keys(cls):
        for obj in cls.objects:
            for fieldname in dir(cls):
                if isinstance(getattr(cls,fieldname), ForeignKey):
                    if not istype(getattr(obj, fieldname), "Model"):
                        setattr(obj, fieldname,\
                                getattr(cls, fieldname).from_raw(getattr(obj, fieldname)))

    @classmethod
    def init_model_class(cls, models_manager):
        """
        Inits model class, sets ModelsManager for Model class
        @param models_manager: ModelsManager object
        """
        cls.__models_manager = models_manager
        cls.views = []
        for method in cls.exclude_methods:
            setattr(cls, method, getattr(Model, method))

        for methodname in dir(cls):
            try:
                if getattr(cls, methodname).method_as_field:
                    setattr(cls, methodname, getattr(cls, methodname).method_as_field)
                    getattr(cls, methodname).always_read_only = True
            except AttributeError:
                pass

        for field in cls.auto_user_fields+\
                     cls.read_only_fields:
            getattr(cls, field).set_read_only_in(cls)

        for fieldname, field in cls.get_rel_fields().items():
            #FIXME!
            try:
                field.model.reverse_sets
            except AttributeError:
                field.model.reverse_sets = {}
            setattr(field.model, cls.__name__.lower()+"_set",
                    lambda inst: cls.filter(**{fieldname:inst}))

    @classmethod
    def flush(cls):
        cls.objects = []
        cls.loaded = False


    @classmethod
    def dump(cls):
        """
        Dumps all new model instances to server
        @return: list of responces dict
        """
        if cls.resource_name is None:
                raise ResourceNameError
        else:
            responces = []
            for o in cls.objects:
                if not o.is_dumped():
                    r = o.to_raw()
                    resp = cls.__models_manager.post_resource_to_server(cls.resource_name, args=r)
                    responces.append(resp)
                    if resp["headers"]["status"]=="200":
                        body = json.loads(resp["body"])
                        o.__refresh(**body)
                        o.__dumped=True
                    elif resp["headers"]["status"]=="400":
                        o.__valid = False

            cls.notify()
            return responces


    def to_raw(self):
        """Returns raw model instance representation"""
        d = {}
        for fieldname, field in self.__class__.get_fields().items():
            if (fieldname!="id") and\
               (fieldname in self.auto_user_fields or not field.is_read_only_in(self.__class__) )\
                    and getattr(self, fieldname) is not None:
                d[fieldname]=unicode(field.to_raw(getattr(self, fieldname))).encode('utf-8')
        from pprint import pprint
        pprint(d)
        return d

    @classmethod
    def all(cls):
        return cls.filter()

    @classmethod
    def new(cls, **kwargs):
        """Used only from user code. Returns new model instance"""
        o = cls(**kwargs)
        o.__dumped = False
        o.id = cls.max_negative_undumped_id()
        cls.objects.append(o)
        return o

    def delete(self):
        """Deletes model instance"""
        self.__class__.objects.remove(self)
        self.notify()
        #FIXME real delete


    @classmethod
    def verbose_name(cls, plural=False):
        """Returns verbose name of model
        @param plural: return plural verbose name"""
        if plural:
            try:
                return cls.Meta.verbose_name_plural
            except AttributeError:
                return cls.__name__
        else:
            try:
                return cls.Meta.verbose_name
            except AttributeError:
                return cls.__name__+"s"

    @classmethod
    def filter(cls, **kwargs):
        return [x for x in cls.objects if x.is_filtered(**kwargs)]

    @classmethod
    def get(cls, id):
        res = cls.filter(id=id)
        if len(res)==1:
            return res[0]

    @classmethod
    def get_fields(cls):
        """
        Returns dict of class Fields           self.refresh()

        """
        f = {}
        for fieldname in dir(cls):
            if istype(getattr(cls, fieldname), "Field"):
                f[fieldname] = getattr(cls, fieldname)
        return f

    @classmethod
    def get_rel_fields(cls):
        """
        Returns dict of relative (Foreign keys, etc) Fields
        """
        f = {}
        for fieldname in dir(cls):
            field = getattr(cls, fieldname)
            try:
                if RelField in getmro(field.__class__):
                    f[fieldname] = getattr(cls, fieldname)
            except AttributeError:
                pass

        return f



    @staticmethod
    def is_model_depend_on(one_class, other_class):
        """
        Returns True, if class has references to another class
        """
#        print one_class , "<=>" , other_class
        for field in one_class.get_rel_fields().values():
            if field.model==other_class:
#                print one_class , "depend on" , other_class
                return 1
        return -1

    @staticmethod
    def is_model_instance_depend_on(model_instance, other_model_instance):
        """
        Returns True, if model instance has references to another class
        """
        for fieldname, field in model_instance.get_rel_fields().items():
            if getattr(model_instance, fieldname)==other_model_instance:
                return 1
        return -1

    def is_filtered(self, **kwargs):
        for field in kwargs:
            try:
                try:
                    if not getattr(self, field) in kwargs[field]:
                        return False
                except TypeError:
                    if not getattr(self, field)==kwargs[field]:
                        return False
            except AttributeError:
                if "__" in field:
                    fs = field.split("__")
                    m = self
                    for i,f in enumerate(fs):
                        try:
                            m=getattr(m,f)
                        except AttributeError:
                            return False
                    if kwargs[field]!=m:
                        return False
                else:
                    print field, self
                    raise KeyError
        return True

    def is_dumped(self):
        """
        Returns True if model instance was dumped to server or was fetched from server
        """
        return self.__dumped

    def is_valid(self):
        """
        Returns False if model instance failed validation
        """
        return self.__valid


    def validate(self):
        result = {}
        for fieldname, field in self.get_fields().items():
            result[fieldname]=field.validate(getattr(self, fieldname), self)
        return result


    @classmethod
    def is_all_dumped(cls):
        """
        Returns True if all Model instances is dumped to server
        (there is not unsaved objects)
        """
        for obj in cls.objects:
            if not obj.is_dumped():
                return False
        return True

    def __init__(self, **initdict):
        super(Model,self).__init__()
#        print initdict
        self.__valid = True
        """@ivar __valid: False if model instance failed validation"""
        self.__refresh(**initdict)

    def __refresh(self, **initdict):
        self.__valid = True
        for fieldname, field in self.__class__.get_fields().items():
            try:
                setattr(self, fieldname, field.from_raw(initdict[fieldname]))
            except KeyError:
                setattr(self, fieldname, field.get_blank())



    @classmethod
    def notify(cls):
        """Sends notify to all views, connected to model"""
        print "notify", cls, cls.views
        for v in cls.views:
            v.refresh()
        if not cls.is_all_dumped():
            cls.__models_manager.notify_changes()

    def save(self):
        """
        Saves object
        """
        for user_field_name in self.auto_user_fields:
            setattr(self, user_field_name, self.__models_manager.get_current_user())

        self.__dumped = False
        self.notify()

    @classmethod
    def add_notify(cls, view):
        """docstring for add_notify"""
        cls.views.append(view)

    def __unicode__(self):
        return "default unicode method"

    def extra_to_html(self):
        """
        Returns extra model-related objects, conventered to html
        """
        return ""

    @classmethod
    def max_negative_undumped_id(cls):
        max = 0
        for o in cls.objects:
            if o.id<max:
                max = o.id
        return max+1


class User(Model):
    resource_name = "django/users/"

    username = CharField(u"Имя пользователя", max_length=30, unique=True)
    first_name = CharField(u"Имя", max_length=30, blank=True)
    last_name = CharField(u"Фамилия", max_length=30, blank=True)
    email = EmailField(u"E-mail", blank=True)
    password = CharField(u"Пароль", max_length=128)
    is_staff = BooleanField(u"Статус персонала (доступ к админке)", default=False)
    is_active = BooleanField(u"Активный", default=True)
    is_superuser = BooleanField(u"Статус администратора", default=False)
    last_login = DateTimeField(u"Последний вход")
    date_joined = DateTimeField(u"Дата регистрации")
#    groups = models.ManyToManyField(Group, verbose_name=_('groups'), blank=True,
#       help_text=_("In addition to the permissions manually assigned, this user will also get all permissions granted to each group he/she is in."))
#    user_permissions = models.ManyToManyField(Permission, verbose_name=_('user permissions'), blank=True)
    class Meta:
        verbose_name = u"Пользователь"
        verbose_name_plural = u"Пользователи"

    def __unicode__(self):
        if self.last_name:
            return self.last_name
        else:
            return self.username

if __name__=="__main__":
    
    
#    from cryotec_service.machines.models import *
#    from cryotec_service.clients.models import *
#    Machine.load()
# 
#    print Machine.fields
#    
#    
#    print Machine.resource_name
#    print Client.resource_name
#    print Client.get(1).name
#    print Machine.get(1).serial
#    print Machine.get(1).motohours
    pass



"""
    Copyright (C) 2008 Benjamin O'Steen

    This file is part of python-fedoracommons.

    python-fedoracommons is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    python-fedoracommons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with python-fedoracommons.  If not, see <http://www.gnu.org/licenses/>.
"""

__license__ = 'GPL http://www.gnu.org/licenses/gpl.txt'
__author__ = "Benjamin O'Steen <bosteen@gmail.com>"
__version__ = '0.1'

from google.appengine.api import urlfetch

import urlparse
from urllib import urlencode
import base64
from base64 import encodestring

import re
import md5
import calendar
import time
import random
import sha
import hmac

from mimeTypes import *

import mimetypes

from cStringIO import StringIO



# For Auth implemnentation: Digest (from httplib2)
# TODO: !Important - add proper code attribution for httplib2 parts
USE_WWW_AUTH_STRICT_PARSING = 0
conn = None
# In regex below:
#    [^\0-\x1f\x7f-\xff()<>@,;:\\\"/[\]?={} \t]+             matches a "token" as defined by HTTP
#    "(?:[^\0-\x08\x0A-\x1f\x7f-\xff\\\"]|\\[\0-\x7f])*?"    matches a "quoted-string" as defined by HTTP, when LWS have already been replaced by a single space
# Actually, as an auth-param value can be either a token or a quoted-string, they are combined in a single pattern which matches both:
#    \"?((?<=\")(?:[^\0-\x1f\x7f-\xff\\\"]|\\[\0-\x7f])*?(?=\")|(?<!\")[^\0-\x08\x0A-\x1f\x7f-\xff()<>@,;:\\\"/[\]?={} \t]+(?!\"))\"?
WWW_AUTH_STRICT = re.compile(r"^(?:\s*(?:,\s*)?([^\0-\x1f\x7f-\xff()<>@,;:\\\"/[\]?={} \t]+)\s*=\s*\"?((?<=\")(?:[^\0-\x08\x0A-\x1f\x7f-\xff\\\"]|\\[\0-\x7f])*?(?=\")|(?<!\")[^\0-\x1f\x7f-\xff()<>@,;:\\\"/[\]?={} \t]+(?!\"))\"?)(.*)$")
WWW_AUTH_RELAXED = re.compile(r"^(?:\s*(?:,\s*)?([^ \t\r\n=]+)\s*=\s*\"?((?<=\")(?:[^\\\"]|\\.)*?(?=\")|(?<!\")[^ \t\r\n,]+(?!\"))\"?)(.*)$")
UNQUOTE_PAIRS = re.compile(r'\\(.)')

def _parse_www_authenticate(headers, headername='www-authenticate'):
    """Returns a dictionary of dictionaries, one dict
    per auth_scheme."""
    retval = {}
    if headers.has_key(headername):
        authenticate = headers[headername].strip()
        www_auth = USE_WWW_AUTH_STRICT_PARSING and WWW_AUTH_STRICT or WWW_AUTH_RELAXED
        while authenticate:
            # Break off the scheme at the beginning of the line
            if headername == 'authentication-info':
                (auth_scheme, the_rest) = ('digest', authenticate)                
            else:
                (auth_scheme, the_rest) = authenticate.split(" ", 1)
            # Now loop over all the key value pairs that come after the scheme, 
            # being careful not to roll into the next scheme
            match = www_auth.search(the_rest)
            auth_params = {}
            while match:
                if match and len(match.groups()) == 3:
                    (key, value, the_rest) = match.groups()
                    auth_params[key.lower()] = UNQUOTE_PAIRS.sub(r'\1', value) # '\\'.join([x.replace('\\', '') for x in value.split('\\\\')])
                match = www_auth.search(the_rest)
            retval[auth_scheme.lower()] = auth_params
            authenticate = the_rest.strip()
    return retval

def _cnonce():
    dig = md5.new("%s:%s" % (time.ctime(), ["0123456789"[random.randrange(0, 9)] for i in range(20)])).hexdigest()
    return dig[:16]

def _wsse_username_token(cnonce, iso_now, password):
    return base64.encodestring(sha.new("%s%s%s" % (cnonce, iso_now, password)).digest()).strip()

URI = re.compile(r"^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?")

def parse_uri(uri):
    """Parses a URI using the regex given in Appendix B of RFC 3986.

        (scheme, authority, path, query, fragment) = parse_uri(uri)
    """
    groups = URI.match(uri).groups()
    return (groups[1], groups[3], groups[4], groups[6], groups[8])

# For credentials we need two things, first 
# a pool of credential to try (not necesarily tied to BAsic, Digest, etc.)
# Then we also need a list of URIs that have already demanded authentication
# That list is tricky since sub-URIs can take the same auth, or the 
# auth scheme may change as you descend the tree.
# So we also need each Auth instance to be able to tell us
# how close to the 'top' it is.

class Authentication(object):
    def __init__(self, credentials, host, request_uri, headers, response, content, http):
        (scheme, authority, path, query, fragment) = parse_uri(request_uri)
        self.path = path
        self.host = host
        self.credentials = credentials
        self.http = http

    def depth(self, request_uri):
        (scheme, authority, path, query, fragment) = parse_uri(request_uri)
        return request_uri[len(self.path):].count("/")

    def inscope(self, host, request_uri):
        # XXX Should we normalize the request_uri?
        (scheme, authority, path, query, fragment) = parse_uri(request_uri)
        return (host == self.host) and path.startswith(self.path)

    def request(self, method, request_uri, headers, content):
        """Modify the request headers to add the appropriate
        Authorization header. Over-rise this in sub-classes."""
        pass

    def response(self, response, content):
        """Gives us a chance to update with new nonces
        or such returned from the last authorized response.
        Over-rise this in sub-classes if necessary.

        Return TRUE is the request is to be retried, for 
        example Digest may return stale=true.
        """
        return False



class BasicAuthentication(Authentication):
    def __init__(self, credentials, host, request_uri, headers, response, content, http):
        Authentication.__init__(self, credentials, host, request_uri, headers, response, content, http)

    def request(self, method, request_uri, headers, content):
        """Modify the request headers to add the appropriate
        Authorization header."""
        headers['authorization'] = 'Basic ' + base64.encodestring("%s:%s" % self.credentials).strip()  


class DigestAuthentication(Authentication):
    """Only do qop='auth' and MD5, since that 
    is all Apache currently implements"""
    def __init__(self, credentials, host, request_uri, headers, response, content, http):
        Authentication.__init__(self, credentials, host, request_uri, headers, response, content, http)
        challenge = _parse_www_authenticate(response, 'www-authenticate')
        self.challenge = challenge['digest']
        qop = self.challenge.get('qop')
        self.challenge['qop'] = ('auth' in [x.strip() for x in qop.split()]) and 'auth' or None
        if self.challenge['qop'] is None:
            raise UnimplementedDigestAuthOptionError( _("Unsupported value for qop: %s." % qop))
        self.challenge['algorithm'] = self.challenge.get('algorithm', 'MD5')
        if self.challenge['algorithm'] != 'MD5':
            raise UnimplementedDigestAuthOptionError( _("Unsupported value for algorithm: %s." % self.challenge['algorithm']))
        self.A1 = "".join([self.credentials[0], ":", self.challenge['realm'], ":", self.credentials[1]])   
        self.challenge['nc'] = 1

    def request(self, method, request_uri, headers, content, cnonce = None):
        """Modify the request headers"""
        H = lambda x: md5.new(x).hexdigest()
        KD = lambda s, d: H("%s:%s" % (s, d))
        A2 = "".join([method, ":", request_uri])
        self.challenge['cnonce'] = cnonce or _cnonce() 
        request_digest  = '"%s"' % KD(H(self.A1), "%s:%s:%s:%s:%s" % (self.challenge['nonce'], 
                    '%08x' % self.challenge['nc'], 
                    self.challenge['cnonce'], 
                    self.challenge['qop'], H(A2)
                    )) 
        headers['Authorization'] = 'Digest username="%s", realm="%s", nonce="%s", uri="%s", algorithm=%s, response=%s, qop=%s, nc=%08x, cnonce="%s"' % (
                self.credentials[0], 
                self.challenge['realm'],
                self.challenge['nonce'],
                request_uri, 
                self.challenge['algorithm'],
                request_digest,
                self.challenge['qop'],
                self.challenge['nc'],
                self.challenge['cnonce'],
                )
        self.challenge['nc'] += 1

    def response(self, response, content):
        if not response.has_key('authentication-info'):
            challenge = _parse_www_authenticate(response, 'www-authenticate').get('digest', {})
            if 'true' == challenge.get('stale'):
                self.challenge['nonce'] = challenge['nonce']
                self.challenge['nc'] = 1 
                return True
        else:
            updated_challenge = _parse_www_authenticate(response, 'authentication-info').get('digest', {})

            if updated_challenge.has_key('nextnonce'):
                self.challenge['nonce'] = updated_challenge['nextnonce']
                self.challenge['nc'] = 1 
        return False


class HmacDigestAuthentication(Authentication):
    """Adapted from Robert Sayre's code and DigestAuthentication above."""
    __author__ = "Thomas Broyer (t.broyer@ltgt.net)"

    def __init__(self, credentials, host, request_uri, headers, response, content, http):
        Authentication.__init__(self, credentials, host, request_uri, headers, response, content, http)
        challenge = _parse_www_authenticate(response, 'www-authenticate')
        self.challenge = challenge['hmacdigest']
        # TODO: self.challenge['domain']
        self.challenge['reason'] = self.challenge.get('reason', 'unauthorized')
        if self.challenge['reason'] not in ['unauthorized', 'integrity']:
            self.challenge['reason'] = 'unauthorized'
        self.challenge['salt'] = self.challenge.get('salt', '')
        if not self.challenge.get('snonce'):
            raise UnimplementedHmacDigestAuthOptionError( _("The challenge doesn't contain a server nonce, or this one is empty."))
        self.challenge['algorithm'] = self.challenge.get('algorithm', 'HMAC-SHA-1')
        if self.challenge['algorithm'] not in ['HMAC-SHA-1', 'HMAC-MD5']:
            raise UnimplementedHmacDigestAuthOptionError( _("Unsupported value for algorithm: %s." % self.challenge['algorithm']))
        self.challenge['pw-algorithm'] = self.challenge.get('pw-algorithm', 'SHA-1')
        if self.challenge['pw-algorithm'] not in ['SHA-1', 'MD5']:
            raise UnimplementedHmacDigestAuthOptionError( _("Unsupported value for pw-algorithm: %s." % self.challenge['pw-algorithm']))
        if self.challenge['algorithm'] == 'HMAC-MD5':
            self.hashmod = md5
        else:
            self.hashmod = sha
        if self.challenge['pw-algorithm'] == 'MD5':
            self.pwhashmod = md5
        else:
            self.pwhashmod = sha
        self.key = "".join([self.credentials[0], ":",
                    self.pwhashmod.new("".join([self.credentials[1], self.challenge['salt']])).hexdigest().lower(),
                    ":", self.challenge['realm']
                    ])
        self.key = self.pwhashmod.new(self.key).hexdigest().lower()

    def request(self, method, request_uri, headers, content):
        """Modify the request headers"""
        keys = _get_end2end_headers(headers)
        keylist = "".join(["%s " % k for k in keys])
        headers_val = "".join([headers[k] for k in keys])
        created = time.strftime('%Y-%m-%dT%H:%M:%SZ',time.gmtime())
        cnonce = _cnonce()
        request_digest = "%s:%s:%s:%s:%s" % (method, request_uri, cnonce, self.challenge['snonce'], headers_val)
        request_digest  = hmac.new(self.key, request_digest, self.hashmod).hexdigest().lower()
        headers['Authorization'] = 'HMACDigest username="%s", realm="%s", snonce="%s", cnonce="%s", uri="%s", created="%s", response="%s", headers="%s"' % (
                self.credentials[0], 
                self.challenge['realm'],
                self.challenge['snonce'],
                cnonce,
                request_uri, 
                created,
                request_digest,
                keylist,
                )

    def response(self, response, content):
        challenge = _parse_www_authenticate(response, 'www-authenticate').get('hmacdigest', {})
        if challenge.get('reason') in ['integrity', 'stale']:
            return True
        return False


class WsseAuthentication(Authentication):
    """This is thinly tested and should not be relied upon.
    At this time there isn't any third party server to test against.
    Blogger and TypePad implemented this algorithm at one point
    but Blogger has since switched to Basic over HTTPS and 
    TypePad has implemented it wrong, by never issuing a 401
    challenge but instead requiring your client to telepathically know that
    their endpoint is expecting WSSE profile="UsernameToken"."""
    def __init__(self, credentials, host, request_uri, headers, response, content, http):
        Authentication.__init__(self, credentials, host, request_uri, headers, response, content, http)

    def request(self, method, request_uri, headers, content):
        """Modify the request headers to add the appropriate
        Authorization header."""
        headers['Authorization'] = 'WSSE profile="UsernameToken"'
        iso_now = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime())
        cnonce = _cnonce()
        password_digest = _wsse_username_token(cnonce, iso_now, self.credentials[1])
        headers['X-WSSE'] = 'UsernameToken Username="%s", PasswordDigest="%s", Nonce="%s", Created="%s"' % (
                self.credentials[0],
                password_digest,
                cnonce,
                iso_now)

class Credentials(object):
    def __init__(self):
        self.credentials = []

    def add(self, name, password, domain=""):
        self.credentials.append((domain.lower(), name, password))

    def clear(self):
        self.credentials = []

    def iter(self, domain):
        for (cdomain, name, password) in self.credentials:
            if cdomain == "" or domain == cdomain:
                yield (name, password) 

AUTH_SCHEME_CLASSES = {
    "basic": BasicAuthentication,
    "Basic": BasicAuthentication,
    "wsse": WsseAuthentication,
    "digest": DigestAuthentication,
    "Digest": DigestAuthentication,
    "hmacdigest": HmacDigestAuthentication
}

AUTH_SCHEME_ORDER = ["hmacdigest", "digest", "Digest", "wsse", "basic", "Basic"]

URLFETCH_METHOD_STRING =   {urlfetch.GET:'GET',
                            urlfetch.PUT:'PUT',
                            urlfetch.DELETE:'DELETE',
                            urlfetch.POST:'POST',
                            urlfetch.HEAD:'HEAD'
                            }


class GAE_Connection:
    def __init__(self, base_url, username=None, password=None):
        self.base_url = base_url
        m = mimeTypes()
        self.mimetypes = m.getDictionary()
        
        # Name/password
        self.credentials = Credentials()
        
        if username and password:
            self.add_credentials(username, password, domain="")

        # authorization objects
        self.authorizations = []
        
        self.url = urlparse.urlparse(base_url)
        
        (scheme, netloc, path, query, fragment) = urlparse.urlsplit(base_url)

        self.scheme = scheme
        self.host = netloc
        self.path = path

    def _auth_from_challenge(self, host, request_uri, headers, response, content):
        """A generator that creates Authorization objects
           that can be applied to requests.
        """
        challenges = _parse_www_authenticate(response, 'www-authenticate')
        for cred in self.credentials.iter(host):
            for scheme in AUTH_SCHEME_ORDER:
                if challenges.has_key(scheme):
                    yield AUTH_SCHEME_CLASSES[scheme](cred, host, request_uri, headers, response, content, self)

    def add_credentials(self, name, password, domain=""):
        """Add a name and password that will be used
        any time a request requires authentication."""
        self.credentials.add(name, password, domain)

    def clear_credentials(self):
        """Remove all the names and passwords
        that are used for authentication"""
        self.credentials.clear()
        self.authorizations = []
   
    def request_get(self, resource, args = None, headers={}):
        return self.request(resource, urlfetch.GET, args, headers=headers)
        
    def request_delete(self, resource, args = None, headers={}):
        return self.request(resource, urlfetch.DELETE, args, headers=headers)
        
    def request_post(self, resource, args = None, body = None, filename=None, headers={}):
        return self.request(resource, urlfetch.POST, args , body = body, filename=filename, headers=headers)
        
    def request_put(self, resource, args = None, body = None, filename=None, headers={}):
        return self.request(resource, urlfetch.PUT, args , body = body, filename=filename, headers=headers)
        
    def request_head(self, resource, args = None, body = None, filename=None, headers={}):
        return self.request(resource, urlfetch.HEAD, args , body = body, filename=filename, headers=headers)
        
    def _conn_request(self, conn, request_uri, method, body, headers):
        # Shim to allow easy reuse of httplib2 auth methods - conn param is not used
        urlfetch_response = urlfetch.fetch(request_uri, method=method, payload=body, headers=headers)
        r_headers={'status':urlfetch_response.status_code}
        for header_key in urlfetch_response.headers:
            r_headers[header_key.lower()] = urlfetch_response.headers[header_key]
        
        return (r_headers, urlfetch_response.content.decode('UTF-8'))
        
    def get_content_type(self, filename):
        extension = filename.split('.')[-1]
        guessed_mimetype = self.mimetypes.get(extension, mimetypes.guess_type(filename)[0])
        return guessed_mimetype or 'application/octet-stream'
        
    def request(self, resource, method = urlfetch.GET, args = None, body = None, filename=None, headers={}):
        params = None
        path = resource
        headers['User-Agent'] = 'Basic Agent'
        
        if not headers.get('Content-Type', None):
            headers['Content-Type']='text/plain'
            
        request_path = []
        if self.path != "/":
            if self.path.endswith('/'):
                request_path.append(self.path[:-1])
            else:
                request_path.append(self.path)
            if path.startswith('/'):
                request_path.append(path[1:])
            else:
                request_path.append(path)
        full_path = u'/'.join(request_path)
        
        if args:
            full_path += u"?%s" % (urlencode(args))
            
        request_uri = u"%s://%s%s" % (self.scheme, self.host, full_path)
        
        auths = [(auth.depth(request_uri), auth) for auth in self.authorizations if auth.inscope(host, request_uri)]
        auth = auths and sorted(auths)[0][1] or None
        if auth: 
            auth.request(method, request_uri, headers, body)
        
        (response, content) = self._conn_request(conn, request_uri, method, body, headers)
        
        if auth: 
            if auth.response(response, body):
                auth.request(URLFETCH_METHOD_STRING[method], request_uri, headers, body)
                
                (response, content) = self._conn_request(conn, request_uri, method, body, headers)

        if response['status'] == 401:
            #return {u"body":u"".join(["%s: %s" % (key, response[key]) for key in response])}
            for authorization in self._auth_from_challenge(self.host, request_uri, headers, response, content):
                authorization.request(URLFETCH_METHOD_STRING[method], request_uri, headers, body) 
                
                (response, content) = self._conn_request(conn, request_uri, method, body, headers)
                
                if response['status'] != 401:
                    self.authorizations.append(authorization)
                    authorization.response(response, body)
                    break
        
        return {u'headers':response, u'body':content}







import sys
import os
import pickle
from helpers import get_all_models
from qtdjango.models import Model
from qtdjango.connection import Connection

class ModelsManager(object):
    """
    Models Manager class - loads models, save thems, etc...
    """

    def __init__(self, server, url, path_to_django_project, app_list, exclude_model_names=None,\
            login=None, password=None):
        """
        @param server: address of django server
        @param url: url path to django server api
        @param path_to_django: path to django project
        @param app_list: list of apps to process
        @param exclude_models: list to models which is not process
        @param login: login to django server
        @param password: password to django server
        """
        self.models = self.__get_registered_models(path_to_django_project,\
                app_list, exclude_model_names)



        self.notify_dumped = []
        """@ivar notify_dumped: list of functions, that calls when all saved"""

        self.notify_undumped = []
        """@ivar notify_undumped: list of functions, that calls when changes"""

        self.notify_user_changed = []

        self.models.sort(cmp=Model.is_model_depend_on)

        for m in self.models:
            m.version = 0
            m.init_model_class(models_manager=self)

        self.set_connection_params(server, url, login, password)


    def set_connection_params(self, server, url, login, password):

        self.__connection = Connection(server, url, login, password)
        current_user = self.get_current_user()
        for func in self.notify_user_changed:
            func(current_user)

    def load_from_server(self):
        try:
            for model in self.models:
                model.flush()
            for model in self.models:
                model.version += 1

                model.load()
            for model in self.models:
                model.refresh_foreing_keys()
            for model in self.models:
                model.notify()
        except:
            return False
        finally:
            for func in self.notify_dumped:
                func()
        return True

    def save_to_file(self, file):
        """
        Saves current modelmanager state to file
        """
        objs = {}
        for model in self.models:
            objs[model.__name__] = model.objects
        pickle.dump(objs, file)

    def load_from_file(self, file):
        """
        Loads modelmanager state from file
        """
        objs = pickle.load(file)

        for model in self.models:
            model.objects = objs[model.__name__]
            model.notify()
            print model.all()
        if not self.is_all_dumped():
            self.notify_changes()

        return True


    def add_notify_dumped(self, function):
        """
        Adds function to notify when models dumps list
        """
        self.notify_dumped.append(function)

    def add_notify_undumped(self, function):
        """
        Adds function to notify when models become have changes
        """
        self.notify_undumped.append(function)



    def add_notify_change_user(self, function):
        """
        Adds function to notify when models become have changes
        """
        self.notify_user_changed.append(function)


    def notify_changes(self):
        """
        Notify changes in models
        """
        for func in self.notify_undumped:
            func()

    def get_resource_from_server(self, resource_name):
        """
        Gets responce from remote Django server
        @param resource_name: name of piston resource
        @rtype: dict
        """
        return self.__connection.get_resource_from_server(resource_name)

    def post_resource_to_server(self, resource_name, args):
        """
        Posts to remote Django server
        @param resource_name: name of piston resource
        @param body: body of post request
        @rtype: dict
        """
        return self.__connection.post_resource_to_server(resource_name, args)



    def dump(self):
        """
        Dump all models to server. Notify all functions about it
        @return: responces dict of all models
        """
        responces = {}
        from operator import attrgetter

        self.models.sort(key=attrgetter("dump_order"))
        for model in self.models:
            print model
            responces[model.verbose_name(True)] = model.dump()
        for func in self.notify_dumped:
            func(responces)

    def is_all_dumped(self):
        """
        Returns True, if all models are dumped to server
        (if there is no unsaved changes)
        """
        for model in self.models:
            if not model.is_all_dumped():
                return False
        return True


    def do_models_magic_with_module(self, module):
        """
        Makes module automagicaly look like Django's models.py
        modules
        @param module: imported (via __import__()) module
        """
        for model in self.models:
            setattr(module, model.__name__, model)

    def __getattr__(self, name):
        try:
            return super(ModelsManager,self).__getattr__(name)
        except AttributeError:
            for model in self.models:
                if model.__name__==name:
                    return model
            raise AttributeError

    def __get_registered_models(self, path_to_django_project, app_list,
                              exclude_model_names=None):
        """
        Gets models from django project by parametres,
        @rtype: list of Models
        """
        if os.path.exists(path_to_django_project):
            path = path_to_django_project
        else:
            try:
                p = __import__(path_to_django_project)
                path = os.path.dirname(p.__file__)
            except ImportError:
                raise ImportError


        sys.path.append(path)

        ms = get_all_models(app_list, from_django=False,\
                            exclude_model_names=exclude_model_names)

        return ms


    def get_current_user(self):
        user_name = self.__connection.get_login_password()[0]
        users = self.User.filter(username=user_name)
        if len(users)==1:
            return users[0]
        elif len(users)==0:
            return None
        else:
            raise AssertionError



"""
iri2uri

Converts an IRI to a URI.

"""
__author__ = "Joe Gregorio (joe@bitworking.org)"
__copyright__ = "Copyright 2006, Joe Gregorio"
__contributors__ = []
__version__ = "1.0.0"
__license__ = "MIT"
__history__ = """
"""

import urlparse


# Convert an IRI to a URI following the rules in RFC 3987
# 
# The characters we need to enocde and escape are defined in the spec:
#
# iprivate =  %xE000-F8FF / %xF0000-FFFFD / %x100000-10FFFD
# ucschar = %xA0-D7FF / %xF900-FDCF / %xFDF0-FFEF
#         / %x10000-1FFFD / %x20000-2FFFD / %x30000-3FFFD
#         / %x40000-4FFFD / %x50000-5FFFD / %x60000-6FFFD
#         / %x70000-7FFFD / %x80000-8FFFD / %x90000-9FFFD
#         / %xA0000-AFFFD / %xB0000-BFFFD / %xC0000-CFFFD
#         / %xD0000-DFFFD / %xE1000-EFFFD

escape_range = [
   (0xA0, 0xD7FF ),
   (0xE000, 0xF8FF ),
   (0xF900, 0xFDCF ),
   (0xFDF0, 0xFFEF),
   (0x10000, 0x1FFFD ),
   (0x20000, 0x2FFFD ),
   (0x30000, 0x3FFFD),
   (0x40000, 0x4FFFD ),
   (0x50000, 0x5FFFD ),
   (0x60000, 0x6FFFD),
   (0x70000, 0x7FFFD ),
   (0x80000, 0x8FFFD ),
   (0x90000, 0x9FFFD),
   (0xA0000, 0xAFFFD ),
   (0xB0000, 0xBFFFD ),
   (0xC0000, 0xCFFFD),
   (0xD0000, 0xDFFFD ),
   (0xE1000, 0xEFFFD),
   (0xF0000, 0xFFFFD ),
   (0x100000, 0x10FFFD)
]
 
def encode(c):
    retval = c
    i = ord(c)
    for low, high in escape_range:
        if i < low:
            break
        if i >= low and i <= high:
            retval = "".join(["%%%2X" % ord(o) for o in c.encode('utf-8')])
            break
    return retval


def iri2uri(uri):
    """Convert an IRI to a URI. Note that IRIs must be 
    passed in a unicode strings. That is, do not utf-8 encode
    the IRI before passing it into the function.""" 
    if isinstance(uri ,unicode):
        (scheme, authority, path, query, fragment) = urlparse.urlsplit(uri)
        authority = authority.encode('idna')
        # For each character in 'ucschar' or 'iprivate'
        #  1. encode as utf-8
        #  2. then %-encode each octet of that utf-8 
        uri = urlparse.urlunsplit((scheme, authority, path, query, fragment))
        uri = "".join([encode(c) for c in uri])
    return uri
        
if __name__ == "__main__":
    import unittest

    class Test(unittest.TestCase):

        def test_uris(self):
            """Test that URIs are invariant under the transformation."""
            invariant = [ 
                u"ftp://ftp.is.co.za/rfc/rfc1808.txt",
                u"http://www.ietf.org/rfc/rfc2396.txt",
                u"ldap://[2001:db8::7]/c=GB?objectClass?one",
                u"mailto:John.Doe@example.com",
                u"news:comp.infosystems.www.servers.unix",
                u"tel:+1-816-555-1212",
                u"telnet://192.0.2.16:80/",
                u"urn:oasis:names:specification:docbook:dtd:xml:4.1.2" ]
            for uri in invariant:
                self.assertEqual(uri, iri2uri(uri))
            
        def test_iri(self):
            """ Test that the right type of escaping is done for each part of the URI."""
            self.assertEqual("http://xn--o3h.com/%E2%98%84", iri2uri(u"http://\N{COMET}.com/\N{COMET}"))
            self.assertEqual("http://bitworking.org/?fred=%E2%98%84", iri2uri(u"http://bitworking.org/?fred=\N{COMET}"))
            self.assertEqual("http://bitworking.org/#%E2%98%84", iri2uri(u"http://bitworking.org/#\N{COMET}"))
            self.assertEqual("#%E2%98%84", iri2uri(u"#\N{COMET}"))
            self.assertEqual("/fred?bar=%E2%98%9A#%E2%98%84", iri2uri(u"/fred?bar=\N{BLACK LEFT POINTING INDEX}#\N{COMET}"))
            self.assertEqual("/fred?bar=%E2%98%9A#%E2%98%84", iri2uri(iri2uri(u"/fred?bar=\N{BLACK LEFT POINTING INDEX}#\N{COMET}")))
            self.assertNotEqual("/fred?bar=%E2%98%9A#%E2%98%84", iri2uri(u"/fred?bar=\N{BLACK LEFT POINTING INDEX}#\N{COMET}".encode('utf-8')))

    unittest.main()

    



#!/usr/bin/env python
import glob
from distutils.core import setup

"""
py2app/py2exe build script for MyApplication.

Will automatically ensure that all build prerequisites are available
via ez_setup

Usage (Mac OS X):
    python setup.py py2app

Usage (Windows):
    python setup.py py2exe

Usage (Ubuntu/Debian):
    python setup.py --command-packages=stdeb.command bdist_deb

Usage (RPM-based linux distros):
    python setup.py bdist_rpm
"""




execfile('src/qtdjango/__init__.py')




base_options = dict (name='qtdjango',
      install_requires = ["django", ],
      version=__version__,
      description='''Library for connection of PyQt application to Django server,
also reusable app.''',
      author='Sergey Klimov',
      author_email='dcdarv@gmail.com',
      url='http://github.com/darvin/qtdjango',
      package_dir = {'qtdjango': 'src/qtdjango'},
      packages=['qtdjango',
                'qtdjango.restclient',
                'qtdjango.restclient.httplib2',
                'qtdjango.django_qtdjango'],
      license="GPL",
      maintainer="Sergey Klimov",
      maintainer_email="dcdarv@gmail.com",
      classifiers=[
          'Development Status :: 4 - Beta',
          'Operating System :: MacOS :: MacOS X',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX',
          'Programming Language :: Python',
          ],
      
     )

options = base_options 



setup( **options)



# -*- coding: utf-8 -*-

from PyQt4.QtGui import *
from PyQt4 import QtCore, Qt



class AbstractModel(QtCore.QAbstractTableModel):

    def __init__(self, model, filter, fields=None, \
                 parent=None, blank_variant=False, \
                 blank_variant_verbose=u"<пусто>", *args):
        QtCore.QAbstractTableModel.__init__(self, parent, *args)
        self.model = model
        self.filter = filter
        self.blank_variant = blank_variant
        self.blank_variant_verbose = blank_variant_verbose


        if fields =="__unicode__":
            self._without_fields = True
        else:
            self.fields = fields
            self._without_fields = False

    def columnCount(self, parent):
        if self._without_fields:
            return 1
        else:
            return len(self.fields)


    def set_filter(self, filter):
        self.filter = filter
        self.modelReset.emit()


    def filtered_model(self):
        if self.filter is None:
            res = self.model.all()
        else:
            res = self.model.filter(**self.filter)
        if self.blank_variant:
            res = [None,] + res
        return res

    def get_model_instance_by_index(self, index):
        data = self.filtered_model()
        if not index.isValid():
            print index.row()
            return QtCore.QVariant()

        return data[index.row()]

    def get_model_instance_by_int_index(self, index):
        data = self.filtered_model()
        return data[index]

    def get_index_of_model(self, model):
        data = self.filtered_model()
        return data.index(model)

    def get_decorate(self, model_instance):
        """Returns (FontRole, BackgroundRole, ForegroundRole) for model_instance"""

        if model_instance.is_dumped() and model_instance.is_valid():
            return (QtCore.QVariant(), QtCore.QVariant(), QtCore.QVariant())
        else:
            font = QFont()
            font.setItalic(True)
            background = QColor(240, 128, 128)
            if not model_instance.is_valid():
                background = QColor("red")
            foreground = QtCore.QVariant()
            return (font, background, foreground)

    def rowCount(self, parent):
        count = len(self.filtered_model())

        return count

    def headerData(self, section, orientation, role):
        if role != QtCore.Qt.DisplayRole:
            return QtCore.QVariant()
        elif orientation == QtCore.Qt.Vertical:
            return QtCore.QVariant()
        elif orientation == QtCore.Qt.Horizontal:
            if self._without_fields:
                return QtCore.QString.fromUtf8(self.model.verbose_name())
            else:
                return QtCore.QString.fromUtf8(self.model.get_fields()[self.fields[section]].get_label())

class TableModel(AbstractModel):



    def data(self, index, role):
        if not index.isValid():
            return QtCore.QVariant()

        model_instance = self.get_model_instance_by_index(index)
        if role==QtCore.Qt.UserRole:
            return model_instance
        elif role==QtCore.Qt.FontRole:
            return self.get_decorate(model_instance)[0]
        elif role==QtCore.Qt.BackgroundRole:
            return self.get_decorate(model_instance)[1]
        elif role==QtCore.Qt.ForegroundRole:
            return self.get_decorate(model_instance)[2]
        elif role==QtCore.Qt.DisplayRole:
            if self._without_fields:
                return model_instance.__unicode__()
            else:
                field_raw_data = getattr(model_instance,self.fields[index.column()])
                result = self.model.get_fields()\
                        [self.fields[index.column()]].to_text(field_raw_data)
                return QtCore.QVariant(result)
        else:
            return QtCore.QVariant()


class ListModel(AbstractModel):
    def columnCount(self, parent):
        return 1

    def data(self, index, role):

        if not index.isValid():
            return QtCore.QVariant()
        else:
            model_instance = self.get_model_instance_by_index(index)
            if role==QtCore.Qt.UserRole:
                return model_instance
            elif role == QtCore.Qt.DisplayRole:
                if model_instance is None:
                    return self.blank_variant_verbose
                else:
                    return QtCore.QVariant(unicode(model_instance))
        return QtCore.QVariant()

class TreeModel(AbstractModel):
    def set_tree_structure(self, tree_structure):
        """@param tree: tree structure dictionary
        {name_of_foreignkey field: model class}"""
        self.__tree = tree_structure


    def parent(self, child):
        node = child.internalPointer()
        if node is None:
            return QModelIndex()
        parent = node.parent
        if parent is None:
            return QModelIndex()
        grandparent = parent.parent
        if grandparent is None:
            return QModelIndex()
        row = grandparent.child.index(parent)
        return self.createIndex(row, 0, parent)

    #def index(self, row, column, parent):
        #if not self.hasIndex(row, column, parent):
            #return QModelIndex()
        #if not parent.isValid():
            #parentItem = self.rootItem
        #else:
            #parentItem = parent.internalPointer()
        #childItem = parentItem.child[row]
        #if childItem:
            #return self.createIndex(row, column, childItem)
        #else:
            #return QtCore.QModelIndex()


    def data(self, index, role):

        if not index.isValid():
            return QtCore.QVariant()
        else:
            model_instance = self.get_model_instance_by_index(index)
            if role==QtCore.Qt.UserRole:
                return model_instance
            elif role == QtCore.Qt.DisplayRole:
                return QtCore.QVariant(unicode(model_instance))
        return QtCore.QVariant()





#from qtdjango.modelsmanager import ModelsManager

#ADDRESS = "http://127.0.0.1:8000"
#API_PATH= "/api/"
#mm = ModelsManager(ADDRESS, API_PATH, "/home/darvin/workspace/cryotec_service/cryotec_service", \
                              #["machines","actions","actiontemplates","clients",],
                              #("Action", "PAction",))

#current_module =__import__(__name__)



#mm.do_models_magic_with_module(current_module)

#print Machine.all()



#import os
#import sys
#from PyQt4.QtGui import *


#from PyQt4.QtCore import *
#from PyQt4.QtGui import *

#some = [
        #[1, "one", [2,3,4]],
        #[2, "tho", [8,]],
        #[3, "thee", [6,7]],
        #[4, "for", [5,]],
        #[5, "five", []],
        #[6, "six", []],
        #[7, "seven", []],
        #[8, "eignt", []],
       
       #]
#class TreeModel(QAbstractItemModel):
    #model = Machine
    #(("machinemark", MachineMark))
    #class zzNode(object):
        #def __init__(self, parent,  uid, txt, childs):
            #super(TreeModel.zzNode, self).__init__()
            #self.parent=parent
            #self.uid=uid
            #self.txt=txt
            #self.child = []
            #for a in childs:
                #self.child.append(TreeModel.zzNode(self, *some[a-1]))
        #def childCount(self):
            #return len(self.child)



    #def __init__(self, parent=None):
        #super(TreeModel, self).__init__(parent)
        #self.columns = 2
        #self.headers = ["key", "value"]
        #self.lastItems = [x for x in some if len(x[2])!=0]
        #print self.lastItems
        #self.rootItem=self.zzNode(None, some[0][0], some[0][1], some[0][2])

    #def rowCount(self, parent):
        #if parent.column() > 0: return 0
        #if not parent.isValid():
            #parentItem = self.rootItem
        #else:
            #parentItem = parent.internalPointer()
        #return parentItem.childCount()

    #def columnCount(self, parent):
        #return self.columns

    #def data(self, index, role):
        #if role == Qt.DisplayRole:
            #tmp=index.internalPointer().child
            #if index.column()==0:
                #return index.internalPointer().uid
            #else:
                #return QVariant(index.internalPointer().txt)
        #else:
            #return QVariant()

    #def headerData(self, section, orientation, role):
        #if orientation == Qt.Horizontal and role == Qt.DisplayRole:
            #return QVariant(self.headers[section])
        #return QVariant()

    #def index(self, row, column, parent):
        #if not self.hasIndex(row, column, parent):
            #return QModelIndex()
        #if not parent.isValid():
            #parentItem = self.rootItem
        #else:
            #parentItem = parent.internalPointer()
        #childItem = parentItem.child[row]
        #if childItem:
            #return self.createIndex(row, column, childItem)
        #else:
            #return QtCore.QModelIndex()

    #def parent(self, child):
        #node = child.internalPointer()
        #if node is None:
            #return QModelIndex()
        #parent = node.parent
        #if parent is None:
            #return QModelIndex()
        #grandparent = parent.parent
        #if grandparent is None:
            #return QModelIndex()
        #row = grandparent.child.index(parent)
        #return self.createIndex(row, 0, parent)


#class TreeOfTableWidget(QTreeView):
    #def __init__(self):
        #super(TreeOfTableWidget, self).__init__()
        #self.setSelectionBehavior(QTreeView.SelectItems)
        #self.setUniformRowHeights(True)
        #model = TreeModel(self)
        #self.setModel(model)

#class MainForm(QMainWindow):
    #def __init__(self):
        #super(MainForm, self).__init__()
        #self.treeWidget = TreeOfTableWidget()
        #self.setCentralWidget(self.treeWidget)
        #QShortcut(QKeySequence("Escape"), self, self.close)
        #self.setWindowTitle("TreeViewExample")

#app = QApplication(sys.argv)




#form = MainForm()
#form.resize(750, 550)
#form.show()
#app.exec_()
